#!/bin/sh
#
# Adds all of the devices listed in /sys/bus/scsi/devices to the iSCSI
# target to be offered up to the initiator.
#

SCSI_TARGET=/proc/scsi_target/scsi_target
DEVICES_DIR=/sys/bus/scsi/devices
MOUNTS_FILE=/etc/mtab
AWK=$(which awk)

if [[ $# -gt 1 || ($# -eq 1 && "$1" != "-f") ]]; then
	echo "Usage:"
	echo "target_add_all_devices.sh [-f]"
	exit 1
fi

test -w $SCSI_TARGET || {
	echo "${SCSI_TARGET} is not writable"
	exit 1
}

for DEVICE in `ls $DEVICES_DIR`
do
	# check if this device is mounted
	test -e ${DEVICES_DIR}/${DEVICE}/block*  && {
		BLOCK_FILE=$(cd ${DEVICES_DIR}/${DEVICE}; ls -d block*)
		DISK=$(echo $BLOCK_FILE | awk -F: '{ print $2 }')
		grep $DISK $MOUNTS_FILE >& /dev/null && {
			if [[ "$1" != "-f" ]]; then
				echo "Skipping $DEVICE mounted as $DISK"
				continue
			else
				echo "Using $DEVICE mounted as $DISK"
			fi
		}
	}

	HOST=$(echo $DEVICE | $AWK -F: '{ print $1 }')
	CHAN=$(echo $DEVICE | $AWK -F: '{ print $2 }')
	ID=$(echo $DEVICE | $AWK -F: '{ print $3 }')
	LUN=$(echo $DEVICE | $AWK -F: '{ print $4 }')

	echo "Adding device $DEVICE"
	echo "adddevice host=$HOST chan=$CHAN id=$ID lun=$LUN" > $SCSI_TARGET
done
