/*	common/debug.h
 * 
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 * 
 *	This file defines the tracing macros and masks
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name IOL and/or UNH may not be used to endorse or promote 
 *	products derived from this software without specific prior 
 * 	written permission.
 */

#ifndef _DEBUG_H
#define _DEBUG_H

#include <linux/types.h>	/* __u32 */

#if defined(__KERNEL__)
#define PRINT printk
#else
#include <stdio.h>
#define PRINT printf
#endif

#if !defined(CONFIG_ISCSI_DEBUG)

#define TRACE_SET(mask) do { } while(0)
#define TRACE_GET(mask) do { } while(0)
#define TRACE_TEST(mask) (0)
#define TRACE(mask, args...) do { } while(0)
#define TRACE_BUFFER(mask, buffer, len, args...) do { } while(0)

#else

#define TRACE_ENDING		0x000000
#define TRACE_DEBUG		0x000001
#define TRACE_ISCSI_FULL	0x000002
#define TRACE_ISCSI		0x000004
#define TRACE_NET		0x000008
#define TRACE_BUF		0x000010
#define TRACE_SEM		0x000020
#define TRACE_ENTER_LEAVE	0x000040
#define TRACE_MY_MEMORY		0x000080
#define TRACE_TIMERS		0x000100
#define TRACE_ERROR_RECOVERY	0x000200
#define TRACE_VERBOSE		0x000400
#define TRACE_ISER		0x000800
#define TRACE_CHAP		0x001000
#define TRACE_SRP		0x002000
#define TRACE_TASK_MGT		0x004000
#define TRACE_NEGOTIATIONS	0x008000
#define TRACE_THREADS		0x010000
#define TRACE_RDMA_EVENTS	0x020000
#define TRACE_ISER_POSTED_RECVS	0x040000
#define TRACE_ISER_MEM          0x080000
#define TRACE_ALL		0xffffff

extern __u32 iscsi_trace_mask;

#define TRACE_SET(mask)		iscsi_trace_mask = mask

#define TRACE_GET(mask)		mask = iscsi_trace_mask

#define TRACE_TEST(mask)	(iscsi_trace_mask & (mask))

#define TRACE(mask, args...)						   \
	do {								   \
		if (iscsi_trace_mask & (mask)) { 			   \
			if (iscsi_trace_mask & TRACE_VERBOSE) { 	   \
				PRINT("%s:%d: ", __FILE__, __LINE__);   \
			}						   \
			PRINT(args);		   			   \
		}							   \
	} while(0)

#define TRACE_BUFFER(mask, buffer, len, args...)			   \
	do {								   \
		if (iscsi_trace_mask & (mask)) { 			   \
			int ndx;					   \
			PRINT(args);					   \
			for (ndx = 0; ndx < len; ndx++)	{		   \
				if ((ndx & 0xf) == 0) {			   \
					PRINT("%3d:", ndx);		   \
				}					   \
				PRINT(" %02x",				   \
					*((__u8 *)(buffer)+ndx)); \
				if ((ndx & 0xf) == 0xf) {		   \
					PRINT("\n");			   \
				}					   \
			}						   \
			if ((ndx & 0xf) != 0) {				   \
				PRINT("\n");				   \
			}						   \
		}							   \
	} while(0)

#endif

#define TRACE_ERROR(args...)						   \
	do  {								   \
		PRINT("%s:%d: ", __FILE__, __LINE__);		   \
		PRINT(__FILE__ ": ***ERROR*** " args);			   \
	} while(0)

#define TRACE_WARNING(args...)						   \
	do  {								   \
		PRINT("***Warning*** " args);				   \
	} while(0)

#endif
