/**
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * \file debugfs.c
 *
 * \author Ethan Burns
 * \date 2008-05-06
 */

#if defined(CONFIG_DEBUG_FS)

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/version.h>

#include "debugfs.h"

/* not sure what verison this really changed in */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,24)
static int debugfs_atomic_t_get(void *data, u64 *val)
{
	*val = atomic_read((atomic_t *) data);
	return 0;
}

static int debugfs_atomic_t_set(void *data, u64 val)
{
	atomic_set((atomic_t *) data, val);
	return 0;
}
#else  /*  */
static u64 debugfs_atomic_t_get(void *data)
{
	return (u64) atomic_read((atomic_t *) data);
}

static void debugfs_atomic_t_set(void *data, u64 val)
{
	atomic_set((atomic_t *) data, val);
}
#endif	/*  */

DEFINE_SIMPLE_ATTRIBUTE(debugfs_atomic_t_fops,
			debugfs_atomic_t_get,
			debugfs_atomic_t_set,
			"%llu\n");

/**
 * Create a debugfs entry for an atomic_t.
 */
struct dentry *debugfs_create_atomic_t(const char *name, mode_t mode,
				       struct dentry *parent,
				       atomic_t *value)
{
	return debugfs_create_file(name, mode, parent, value,
				   &debugfs_atomic_t_fops);
}

#endif /* CONFIG_DEBUG_FS */
