/*	common/ffp_negotiate.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	header file for functions used in the Full Feature phase
 *	by the iSCSI initiator and target for parameter negotiation
 *
 *	Copyright (C) 2001-2005 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote 
 *	products derived from this software without specific prior 
 * 	written permission.
*/

enum text_types { TEXT_EMPTY, TEXT_OTHER, TEXT_ST_ALL, TEXT_ST_TN,
	TEXT_ST_NULL
};

extern __u32 __attribute__ ((no_instrument_function))
    give_text_overflow_message(__u32 newlen, __u32 outmax);


/*
 *	parses data segment sent in a Text Request or Response in FFP
 *	When called,
 *		"inbuf" holds "inlen" bytes of data to be parsed
 *		"p_param_tbl" holds the table of known keys
 *		"discovery_session" is non-zero if this a Discovery Session
 *		"who_sent_it" is USE_BY_INITIATOR if "inbuf" was sent by Initiator,
 *						 USE_BY_TARGET if "inbuf" was sent by Target
 *		"outbuf" is an empty buffer that can hold up to "outmax" bytes
 *				 (or NULL if no reply is being generated)
 *
 *	Returns 0 if ok, else reason code > 0 for reject
 *	Also on return,
 *		if not NULL, "outbuf" has been filled with a response
 *		if not NULL, "length" has been filled with size of response in "outbuf"
 *		"new_max_send_length" is non-zero if MaxRecvDataSegmentLength was recved
 *		if not NULL, "val_string" points at 1st char of string following 'key='
 *  	if not NULL, "result" holds:
 *			TEXT_EMPTY		if no data segment
 *			TEXT_OTHER		if any key other than SendTargets
 *			TEXT_ST_ALL		if only "SendTargets=ALL" key
 *			TEXT_ST_TN		if only "SendTargets=<target-name>" key
 *			TEXT_ST_NULL	if only "SendTargets=" key
 */
extern __u32
parse_text_buffer(char *inbuf, __u32 inlen,
		  struct parameter_type p_param_tbl[MAX_CONFIG_PARAMS],
		  __u32 discovery_session,
		  __u32 who_sent_it,
		  char *outbuf, __u32 outmax,
		  enum text_types *result,
		  __u32 * new_max_send_length, __u32 * length,
		  char **val_string);
