/**
 * \file common/kernel_workqueue.h
 * Abstracts the workqueue differences between various kernel versions
 * \author Ethan Burns
 * \date 2007-11-13
 */
#if defined(__KERNEL__)

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)

/*
 * The argument type to a work function
 */
typedef struct work_struct work_func_arg;

/*
 * Gets the (struct work_struct *) given the work function's argument.
 */
#define GET_WORK_STRUCT_P(arg) (arg)

/*
 * The argument type to a work function
 */
#define KERNEL_INIT_WORK(_work, _func) INIT_WORK(_work, _func)

#else	 /* < 2.6.20 */

/*
 * The argument type to a work function
 */
typedef void work_func_arg;

/*
 * Gets the (struct work_struct *) given the work function's argument.
 */
#define GET_WORK_STRUCT_P(arg) ((struct work_struct *) arg)

/*
 * Initializes a work struct
 */
#define KERNEL_INIT_WORK(_work, _func) INIT_WORK(_work, _func, _work)

#endif	/* >= 2.6.20 else < 2..6.220  */

#endif	/* __KERNEL__ */
/* vi: set tabstop=8 textwidth=72: */
