/**
 * struct scsi_cmnd seem to change a lot.  Here are a bunch of macros to
 * access various fields in various kernel versions.  The version numbers
 * may be wrong for some macros... I will comment
 * them as such below, they should be fixed eventually.
 *
 * @author Ethan A. Burns
 */

#ifndef _UNH_SCSI_CMND_H_
#define _UNH_SCSI_CMND_H_

#if defined(__KERNEL__)

#include <linux/version.h>
#include "scatterlist.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
/*
 * 2.6.25 -- different handling for (what was formerly knows as) the
 * request_buffer and request_bufflen fields of the scsi_cmnd struct.
 */
#define scsi_buffer(cmnd) scsi_sglist(cmnd)
#else			/* < linux-2.6.23 */
#define scsi_buffer(cmnd) (cmnd)->request_buffer
#define scsi_sglist(cmnd) (cmnd)->request_buffer
#define scsi_bufflen(cmnd) (cmnd)->request_bufflen
#define scsi_sg_count(cmnd) (cmnd)->use_sg
#define scsi_get_resid(cmnd) (cmnd)->resid
#endif			/* >= linux-2.6.23 */

/* This version test may not be correct... it may be earlier than
 * 2.6.20 that introduces scsi_for_each_sg() */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
#define scsi_for_each_sg(cmd, ptr, count, index) 	\
	for_each_sg(scsi_sglist(cmd), ptr, count, index)
#endif

#else /* !__KERNEL__ */

/*
 *
 * User-space macros
 *
 */
#define scsi_buffer(cmnd) (cmnd)->request_buffer
#define scsi_sglist(cmnd) (cmnd)->request_buffer
#define scsi_bufflen(cmnd) (cmnd)->request_bufflen
#define scsi_sg_count(cmnd) (cmnd)->use_sg
#define scsi_get_resid(cmnd) (cmnd)->resid

#endif /* __KERNEL__ */

#endif /* !_UNH_SCSI_CMND_H_ */
