/*	common/user_files.c
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the filp_xxx functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#include "user_space.h"

/* in the kernel, this copies a file name from user space into kernel space */
char *getname(char *x)
{
	return x;
}

/* in the kernel, this frees the memory occupied by a file name from getname */
void putname(char *x)
{
}

struct file *filp_open(char *name, int flags, int mode)
{
	struct file *tmp;

	tmp = malloc(sizeof(struct file));
	if (tmp) {
		tmp->fd = open(name, flags, mode);
		if (tmp->fd < 0) {
			free(tmp);
			tmp = NULL;
		}
	}
	return tmp;
}

void filp_close(struct file *fptr, char *x)
{
	if (fptr) {
		close(fptr->fd);
		free(fptr);
	}
}
