/*	common/user_files.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the file functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#ifndef __USER_FILES_H__
#define __USER_FILES_H__


/* for file handling */

#define	IS_ERR(x)		(x) == NULL

#define	PTR_ERR(x)		(errno)

/* skeletons from /usr/src/linux/include/linux/fs.h, modified for user space */
struct file;

#define default_llseek(file,offset,whence)	lseek(file->fd, offset, whence)

/*
#define default_read(file, buffer, nbytes, position)	\
						read(file->fd, buffer, nbytes)
*/

#define default_write(file, buffer, nbytes, position)	\
						write(file->fd, buffer, nbytes)

struct file_operations {
	loff_t(*llseek) (struct file *, loff_t, int);
	ssize_t(*read) (struct file *, char *, size_t, loff_t *);
//      ssize_t (*aio_read) (struct kiocb *, char *, size_t, loff_t);
	ssize_t(*write) (struct file *, const char *, size_t, loff_t *);
//      ssize_t (*aio_write) (struct kiocb *, const char *, size_t, loff_t);
//      int (*readdir) (struct file *, void *, filldir_t);
//      unsigned int (*poll) (struct file *, struct poll_table_struct *);
//      int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
	long (*unlocked_ioctl) (struct file *, unsigned int,
				unsigned long);
	long (*compat_ioctl) (struct file *, unsigned int, unsigned long);
//      int (*mmap) (struct file *, struct vm_area_struct *);
//      int (*open) (struct inode *, struct file *);
	int (*flush) (struct file *);
//      int (*release) (struct inode *, struct file *);
//      int (*fsync) (struct file *, struct dentry *, int datasync);
//      int (*aio_fsync) (struct kiocb *, int datasync);
	int (*fasync) (int, struct file *, int);
//      int (*lock) (struct file *, int, struct file_lock *);
	 ssize_t(*readv) (struct file *, const struct iovec *,
			  unsigned long, loff_t *);
	 ssize_t(*writev) (struct file *, const struct iovec *,
			   unsigned long, loff_t *);
//      ssize_t (*sendfile) (struct file *, loff_t *, size_t, read_actor_t, void *);
//      ssize_t (*sendpage) (struct file *, struct page *, int, size_t, loff_t *, int);
	unsigned long (*get_unmapped_area) (struct file *, unsigned long,
					    unsigned long, unsigned long,
					    unsigned long);
	int (*check_flags) (int);
	int (*dir_notify) (struct file * filp, unsigned long arg);
//      int (*flock) (struct file *, int, struct file_lock *);
//      ssize_t (*splice_write)(struct pipe_inode_info *, struct file *, loff_t *, size_t, unsigned int);
//      ssize_t (*splice_read)(struct file *, loff_t *, struct pipe_inode_info *, size_t, unsigned int);
};

struct file {
	int fd;
	const struct file_operations *f_op;
	loff_t f_pos;
};

/* in the kernel, this copies a file name from user space into kernel space */
extern char *getname(char *x);

/* in the kernel, this frees the memory occupied by a file name from getname */
extern void putname(char *x);

extern struct file *filp_open(char *name, int flags, int mode);

extern void filp_close(struct file *fptr, char *x);

#endif
