/*	common/user_scsi.h -	user space version of
 *				/usr/src/linux/include/scsi/scsi.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the SCSI functions and data structures used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#ifndef __USER_SCSI_H__
#define __USER_SCSI_H__

/* user space version of  /usr/src/linux/include/scsi/scsi.h */

/*
 *	SCSI opcodes
 */

#define	READ_BLOCK_LIMITS	0x05
#define	INQUIRY			0x12
#define	READ_CAPACITY		0x25

/*
 *  Status codes. These are deprecated as they are shifted 1 bit right
 *  from those found in the SCSI standards. This causes confusion for
 *  applications that are ported to several OSes. Prefer SAM Status codes
 *  above.
 */

#define GOOD                 0x00
#define CHECK_CONDITION      0x01
#define CONDITION_GOOD       0x02
#define BUSY                 0x04
#define INTERMEDIATE_GOOD    0x08
#define INTERMEDIATE_C_GOOD  0x0a
#define RESERVATION_CONFLICT 0x0c
#define COMMAND_TERMINATED   0x11
#define QUEUE_FULL           0x14
#define ACA_ACTIVE           0x18
#define TASK_ABORTED         0x20

#ifndef STATUS_MASK
#define	STATUS_MASK	0xfe
#endif

/*
 *  SENSE KEYS
 */

#define NO_SENSE            0x00
#define RECOVERED_ERROR     0x01
#define NOT_READY           0x02
#define MEDIUM_ERROR        0x03
#define HARDWARE_ERROR      0x04
#define ILLEGAL_REQUEST     0x05
#define UNIT_ATTENTION      0x06
#define DATA_PROTECT        0x07
#define BLANK_CHECK         0x08
#define COPY_ABORTED        0x0a
#define ABORTED_COMMAND     0x0b
#define VOLUME_OVERFLOW     0x0d
#define MISCOMPARE          0x0e

/*
 * Host byte codes
 */

#define	DID_OK		0x00	/* NO error                               */
#define	DID_NO_CONNECT	0x01	/* Couldn't connect before timeout period */
#define	DID_ABORT	0x05	/* Told to abort for some other reason    */
#define	DID_ERROR	0x07	/* Internal error                         */

/*
 *  These indicate the error that occurred, and what is available.
 */

#define DRIVER_BUSY         0x01
#define DRIVER_SOFT         0x02
#define DRIVER_MEDIA        0x03
#define DRIVER_ERROR        0x04

#define DRIVER_INVALID      0x05
#define DRIVER_TIMEOUT      0x06
#define DRIVER_HARD         0x07
#define DRIVER_SENSE        0x08

/*
 * Internal return values.
 */

#define	SUCCESS		0x2002
#define	FAILED		0x2003

/*
 *  Use these to separate status msg and our bytes
 *
 *  These are set by:
 *
 *      status byte = set from target device
 *      msg_byte    = return status from host adapter itself.
 *      host_byte   = set by low-level driver to indicate status.
 *      driver_byte = set by mid-level.
 */
#define status_byte(result) (((result) >> 1) & 0x7f)
#define msg_byte(result)    (((result) >> 8) & 0xff)
#define host_byte(result)   (((result) >> 16) & 0xff)
#define driver_byte(result) (((result) >> 24) & 0xff)
#define suggestion(result)  (driver_byte(result) & SUGGEST_MASK)


/* from /usr/src/linux/include/scsi/scsi_cmnd.h */
#define	MAX_COMMAND_SIZE	16
#define SCSI_SENSE_BUFFERSIZE	96

/* from /usr/src/linux/include/linux/dma-mapping.h */
enum dma_data_direction {
	DMA_BIDIRECTIONAL = 0,
	DMA_TO_DEVICE = 1,
	DMA_FROM_DEVICE = 2,
	DMA_NONE = 3,
};

#ifndef SCSI_DATA_UNKNOWN
#define SCSI_DATA_UNKNOWN	(DMA_BIDIRECTIONAL)
#endif
#ifndef SCSI_DATA_WRITE
#define SCSI_DATA_WRITE		(DMA_TO_DEVICE)
#endif
#ifndef SCSI_DATA_READ
#define SCSI_DATA_READ		(DMA_FROM_DEVICE)
#endif
#ifndef SCSI_DATA_NONE
#define SCSI_DATA_NONE		(DMA_NONE)
#endif

#endif
