/*	common/user_semaphore.c -	user space version of
 *					/usr/src/linux/include/asm/sempahore.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the user-level interface to semaphores.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#include "user_space.h"

/**
 * Initialize a semaphore to a given value
 */
void sema_init(struct semaphore *s, int val)
{
	pthread_mutex_init(&s->mutex, NULL);
	pthread_cond_init(&s->cond, NULL);
	s->count.counter = val;
}

void init_MUTEX_LOCKED(struct semaphore *s)
{
	sema_init(s, 0);
}

void init_MUTEX_UNLOCKED(struct semaphore *s)
{
	sema_init(s, 1);
}

int down_interruptible(struct semaphore *s)
{
	TRACE(TRACE_ENTER_LEAVE, "Entering %s semaphore %p\n", __func__, s);

	pthread_mutex_lock(&s->mutex);
	while (atomic_read(&s->count) <= 0)
		pthread_cond_wait(&s->cond, &s->mutex);
	atomic_dec(&s->count);
	pthread_mutex_unlock(&s->mutex);

	TRACE(TRACE_ENTER_LEAVE, "Leaving %s semaphore %p\n", __func__, s);
	return 0;
}

void down(struct semaphore *s)
{
	TRACE(TRACE_ENTER_LEAVE, "Entering %s sempahore %p\n", __func__, s);

	down_interruptible(s);

	TRACE(TRACE_ENTER_LEAVE, "Leaving %s semaphore %p\n", __func__, s);
}

void up(struct semaphore *s)
{
	TRACE(TRACE_ENTER_LEAVE, "Entering %s sempahore %p\n", __func__, s);

	pthread_mutex_lock(&s->mutex);
	atomic_inc(&s->count);
	pthread_cond_signal(&s->cond);
	pthread_mutex_unlock(&s->mutex);

	TRACE(TRACE_ENTER_LEAVE, "Leaving %s semaphore %p\n", __func__, s);
}
