/*	common/user_signal.c -	user space version of
 *				/usr/src/linux/include/linux/signal.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the signal functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#include "user_space.h"
#include <sched.h>

/* from /usr/src/linux/include/linux/signal.h */

/* called to unblock all the signals whose bits are set in sig_bit_mask */
void siginitsetinv(sigset_t * set, unsigned long sig_bit_mask)
{
	sigset_t temp_set;
	int sig;

	/* use sig_bit_mask to initialize temp_set */
	sigemptyset(&temp_set);
	for (sig = 1; sig_bit_mask != 0; sig_bit_mask >>= 1, sig++) {
		if (sig_bit_mask & 1)
			sigaddset(&temp_set, sig);
	}
	sigprocmask(SIG_UNBLOCK, &temp_set, NULL);
}

/* returns true if any signals whose bits are set in sig_bit_mask are pending */
int sigtestsetmask(sigset_t * set, unsigned long sig_bit_mask)
{
	sigset_t temp_set;
	int sig;

	sigpending(&temp_set);
	for (sig = 1; sig_bit_mask != 0; sig_bit_mask >>= 1, sig++) {
		if (sig_bit_mask & 1)
			return 1;
	}
	return 0;
}

/* removes pending signal sig */
void sigdelsetmask(sigset_t * set, unsigned long sig)
{
	sigset_t temp_set;
	int real_sig;

	/* remove sig from pending signals */
	sigemptyset(&temp_set);
	sigaddset(&temp_set, sig);
	sigwait(&temp_set, &real_sig);
}

/* sends a signal to a task */
int send_sig(int sig, struct task_struct *task, int value)
{
	int err;
	struct task_struct *current;

	current = pthread_getspecific(current_key);

	TRACE(TRACE_THREADS,
	      "%s, thread id %#08lx, sending signal %d to %s, "
	      "thread id %#08lx\n", current->comm, current->threadid, sig,
	      task->comm, task->threadid);

	if ((err = pthread_cancel(task->threadid)) != 0) {
		TRACE_ERROR
		    ("%s, thread id %#08lx, cancel %s thread id %#08lx: "
		     "%s\n", current->comm, current->threadid, task->comm,
		     task->threadid, strerror(err));
	} else {		/* give the cancelled task's cleanup handlers a chance to run */
		sched_yield();
	}
	return 0;
}
