/*	common/user_sockets.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the socket functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#ifndef __USER_SOCKETS_H__
#define __USER_SOCKETS_H__

/* kernel versions are in <linux/net.h> */

struct socket;

struct proto_ops {
	int (*release) (struct socket * sock);
	int (*bind) (struct socket * sock,
		     struct sockaddr * myaddr, int sockaddr_len);
	int (*connect) (struct socket * sock,
			struct sockaddr * vaddr,
			int sockaddr_len, int flags);
	int (*socketpair) (struct socket * sock1, struct socket * sock2);
	int (*accept) (struct socket * sock,
		       struct socket * newsock, int flags);
	int (*getname) (struct socket * sock,
			struct sockaddr * addr,
			int *sockaddr_len, int peer);
//      unsigned int    (*poll)      (struct file *file, struct socket *sock,
//                                    struct poll_table_struct *wait);
	int (*ioctl) (struct socket * sock, unsigned int cmd,
		      unsigned long arg);
//      int             (*compat_ioctl) (struct socket *sock, unsigned int cmd,
//                                    unsigned long arg);
	int (*listen) (struct socket * sock, int backlog);
	int (*shutdown) (struct socket * sock, int flags);
	int (*setsockopt) (struct socket * sock, int level,
			   int optname, void *optval, int optlen);
	int (*getsockopt) (struct socket * sock, int level,
			   int optname, void *optval, int *optlen);
//      int             (*compat_setsockopt)(struct socket *sock, int level,
//                                    int optname, char *optval, int optlen);
//      int             (*compat_getsockopt)(struct socket *sock, int level,
//                                    int optname, char *optval, int *optlen);
	int (*sendmsg) (void *iocb, struct socket * sock,
			struct msghdr * m, size_t total_len);
	int (*recvmsg) (void *iocb, struct socket * sock,
			struct msghdr * m, size_t total_len, int flags);
//      int             (*mmap)      (struct file *file, struct socket *sock,
//                                    struct vm_area_struct * vma);
//      ssize_t         (*sendpage)  (struct socket *sock, struct page *page,
//                                    int offset, size_t size, int flags);
};

/**
 *  struct socket - general BSD socket
 *  @state: socket state (%SS_CONNECTED, etc)
 *  @flags: socket flags (%SOCK_ASYNC_NOSPACE, etc)
 *  @ops: protocol specific socket operations
 *  @fasync_list: Asynchronous wake up list
 *  @file: File back pointer for gc
 *  @sk: internal networking protocol agnostic socket representation
 *  @wait: wait queue for several uses
 *  @type: socket type (%SOCK_STREAM, etc)
 */
struct socket {
	int fd;
	int type;
	const struct proto_ops *ops;	/* the jump table */
	struct sock *sk;
};

extern int sock_create(int family, int type, int protocol,
		       struct socket **sockptr);

extern void *sock_alloc(void);
extern int sock_release(struct socket *sockptr);

extern int sock_sendmsg(struct socket *sockptr, struct msghdr *msg,
			int length);
extern int sock_recvmsg(struct socket *sockptr, struct msghdr *msg,
			int length, int flags);

#endif
