/*	common/user_threads.c
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the threads functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#include "user_space.h"

/* for thread handling */


/* from /usr/src/linux/include/asm/processor.h */
/*
 * \bug This should return a `pid' for the new thread, however, the threadid
 *      returned from pthread_create can be large enough to be negative when
 *      cast into an int (this would be interpreted as an error by the
 *      caller).
 */
pid_t kernel_thread(int (*fn) (void *), void *arg, unsigned long flags)
{
	pthread_t threadid;
	int errcode;
	struct task_struct *current;

	current = pthread_getspecific(current_key);

	TRACE(TRACE_THREADS,
	      "%s kernel_thread called from thread id %#08lx\n",
	      current->comm, pthread_self());
	errcode =
	    pthread_create(&threadid, NULL, (void *(*)(void *)) fn, arg);
	if (errcode) {
		TRACE_ERROR("pthread_create: %s\n", strerror(errcode));
		exit(EXIT_FAILURE);
	}
	TRACE(TRACE_THREADS,
	      "%s kernel_thread created child with thread id %#08lx\n",
	      current->comm, (long) threadid);
	return 1;
}


/* cleanup handler for the global key,
 * called on the exit of thread having non-null value of its private "current"
 */
void free_current(void *current)
{
	TRACE(TRACE_THREADS, "thread id %#08lx freeing current %p\n",
	      pthread_self(), current);
	free(current);
}


void daemonize(void)
{
	int err;
	struct task_struct *current;

	TRACE(TRACE_THREADS, "start daemonize from thread id %#08lx\n",
	      pthread_self());

	if ((current = calloc(1, sizeof(struct task_struct))) == NULL) {
		TRACE_ERROR("daemonize calloc current: %s\n",
			    strerror(errno));
		exit(EXIT_FAILURE);
	}
	TRACE(TRACE_THREADS, "thread id %#08lx allocing current %p\n",
	      pthread_self(), current);

	current->pid = getpid();
	current->threadid = pthread_self();
	sigemptyset(&current->blocked);
	if ((err = pthread_setspecific(current_key, current)) != 0) {
		TRACE_ERROR("daemonize setspecific current_key: %s\n",
			    strerror(err));
		exit(EXIT_FAILURE);
	}

	/*
	pthread_detach(pthread_self());
	*/

	TRACE(TRACE_THREADS, " done daemonize from thread id %#08lx\n",
	      pthread_self());
}
