/*	common/user_timer.c
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This defines the timer functions used
 *	by both the iSCSI target and the iSCSI initiator
 *	to run in user space instead of in the kernel.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/

#include "user_space.h"

int timer_pending(const struct timer_list *timer)
{
	return (timer->timer_active > 0);
}

void add_timer(struct timer_list *timer)
{
	timer->timer_active = 1;
}

int del_timer_sync(struct timer_list *timer)
{
	TRACE(TRACE_TIMERS, "%s: timerid=%lu\n",
	      __func__, (unsigned long) timer->timerid);
	timer_delete(timer->timerid);
	timer->timer_active = 0;
	return 0;
}

int del_timer(struct timer_list *timer)
{
	TRACE(TRACE_TIMERS, "%s: timerid=%lu\n",
	      __func__, (unsigned long) timer->timerid);
	timer_delete(timer->timerid);
	timer->timer_active = 0;
	return 0;
}

void init_timer(struct timer_list *timer)
{
	timer->timer_active = 0;
	if (timer_create(CLOCK_REALTIME, NULL, &timer->timerid)) {
		perror("timer create");
		timer->timer_active = -1;
	}
	TRACE(TRACE_TIMERS, "%s: timerid=%lu\n",
	      __func__, (unsigned long) timer->timerid);
}
