/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_TYPES_H__
#define __ISER_TYPES_H__

/*! --------------------------------------------------------------------
   [iser_status_t]

   Description:
    Return code for all iSER API functions
 -------------------------------------------------------------------- */
typedef enum {
	ISER_SUCCESS = 0,	/* operation successful */
	ISER_FAILURE,		/* operation failed */
	ISER_INVALID_CONN,	/* invalid connection handle supplied */
	ISER_INVALID_ITT,	/* invalid Initiator Task Tag supplied */
	ISER_ILLEGAL_PARAM,	/* illegal value of a parameter, e.g. NULL pointer */
	ISER_UNREGISTERED,	/* this function has not been registered */
	ISER_INVALID_KEY,	/* Unsupported or invalid key passed to Notice_Key_Value */
	ISER_INVALID_VALUE,	/* Unsupported or invalid value passed to Notice_Key_Value */
} iser_status_t;

/*! --------------------------------------------------------------------
   [iser_entity_type]

   Description:
    Type of iSCSI entity interfacing iSER
 -------------------------------------------------------------------- */
enum iser_entity_type {
	ISER_ENTITY_INITIATOR = 0,
	ISER_ENTITY_TARGET,

	iSER_ENTITY_TYPES_NUM
};

/*! --------------------------------------------------------------------
   [iser_api_handle_t]

   Description:
    API registration handle
 -------------------------------------------------------------------- */
typedef void *iser_api_handle_t;

#define ISER_INVALID_API_H      ((iser_api_handle_t)~0x0)

/*! --------------------------------------------------------------------
   [iser_conn_handle_t]

   Description:
    iSCSI or iSER supplied connection handle 
 -------------------------------------------------------------------- */
typedef void *iser_conn_handle_t;

/*! --------------------------------------------------------------------
   [iser_transport_conn_t]

   Description:
    Transport connection descriptor, e.g. struct socket* for INET 
 -------------------------------------------------------------------- */
typedef void *iser_transport_conn_t;

/*! --------------------------------------------------------------------
   [iser_transport_conn_type]

   Description:
    Transport connection type
 -------------------------------------------------------------------- */
enum iser_transport_conn_type {
	ISER_TRANSPORT_CONN_INET = 0,
	ISER_TRANSPORT_CONN_IB,
};

/*! --------------------------------------------------------------------
   [iser_buf_type]

   Description:
    Type of buffer representation
 -------------------------------------------------------------------- */
enum iser_buf_type {
	ISER_BUF_TYPE_SINGLE = 0,	/* single contiguous buffer, virtual address */
	ISER_BUF_TYPE_IOVEC_VIRT,	/* struct iovec array, virtual addresses */
	ISER_BUF_TYPE_SCATTERLIST,	/* struct scatterlist array */

	ISER_BUF_TYPES_NUM
};

/*! --------------------------------------------------------------------
   [iser_data_buf_t]
 
   Description: 
    Data buffer descriptor. 
    May describe both plain single-chunk buffers and scatter-gather lists.
    To keep this API-related header file more or less constant in face 
    of possible changes in the platform-specific data structures used, 
    iser_data_buf_t has two generic fields: p_buf and size.
    If only plain buffers are used, then p_buf is a pointer to a memory 
    location and size is the size of the buffer.
    If scatter-gather lists are used, p_buf points to the vector of 
    scatter-gather entries and size is the vector's size.
    Specific type of buffer representation is passed in the "type" field.
 -------------------------------------------------------------------- */
struct iser_data_buf_t {

	void *p_buf;
	unsigned int size;
	enum iser_buf_type type;

};				/* iser_data_buf_t */

#endif				/* __ISER_TYPES_H__ */
