/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_BHS_H__
#define __ISER_BHS_H__

/* -------------------------------------------------------- 
 * opcode byte
 * --------------------------------------------------------
 */

/* Set when marked for immediate delivery */
#define ISCSI_OP_IMMEDIATE              0x40
/* Masks out the opcode itself */
#define ISCSI_OPCODE_MASK               0x3F

/* Initiator only opcode values */
#define ISCSI_OP_NOP_OUT                0x00
#define ISCSI_OP_SCSI_CMD               0x01
#define ISCSI_OP_TASK_MGT_REQ           0x02
#define ISCSI_OP_LOGIN_REQ              0x03
#define ISCSI_OP_TEXT_REQ               0x04
#define ISCSI_OP_DATA_OUT               0x05
#define ISCSI_OP_LOGOUT_REQ             0x06
#define ISCSI_OP_SNACK_REQ              0x10

/* Target only opcode values */
#define ISCSI_OP_NOP_IN                 0x20
#define ISCSI_OP_SCSI_RSP               0x21
#define ISCSI_OP_TASK_MGT_RSP           0x22
#define ISCSI_OP_LOGIN_RSP              0x23
#define ISCSI_OP_TEXT_RSP               0x24
#define ISCSI_OP_DATA_IN                0x25
#define ISCSI_OP_LOGOUT_RSP             0x26
#define ISCSI_OP_R2T                    0x31
#define ISCSI_OP_ASYNC                  0x32
#define ISCSI_OP_REJECT                 0x3f

/* -------------------------------------------------------- 
 * flags byte
 * -------------------------------------------------------- 
 */

/* When set indicates the final (or only) PDU of a sequence */
#define ISCSI_FLAG_FINAL                0x80
/* When set indicates that data is to be read */
#define ISCSI_FLAG_READ_CMD             0x40
/* When set indicates that data is to be written */
#define ISCSI_FLAG_WRITE_CMD            0x20

#define IS_SET_ISCSI_FLAG_FINAL(x) \
                (ISCSI_FLAG_FINAL & x)

#define IS_SET_ISCSI_FLAG_READ_CMD(x) \
                (ISCSI_FLAG_READ_CMD & x)

#define IS_SET_ISCSI_FLAG_WRITE_CMD(x) \
                (ISCSI_FLAG_WRITE_CMD & x)

/* Login specific */
#define ISCSI_FLAG_LOGIN_TRANSIT        0x80
#define ISCSI_FLAG_LOGIN_CONTINUE       0x40

#define IS_SET_ISCSI_FLAG_LOGIN_TRANSIT(x) \
                (ISCSI_FLAG_LOGIN_TRANSIT & x)

#define IS_SET_ISCSI_FLAG_LOGIN_CONTINUE(x) \
                (ISCSI_FLAG_LOGIN_CONTINUE & x)

#define ISCSI_FLAG_LOGIN_CSG_MASK       0x0C
#define ISCSI_FLAG_LOGIN_NSG_MASK       0x03

#define GET_ISCSI_FLAG_LOGIN_CSG(x) \
                (ISCSI_FLAG_LOGIN_CSG_MASK & x)

#define GET_ISCSI_FLAG_LOGIN_NSG(x) \
                (ISCSI_FLAG_LOGIN_NSG_MASK & x)

#define ISCSI_LOGIN_STAGE_SECURITY      0x00
#define ISCSI_LOGIN_STAGE_OPERATIONAL   0x01
#define ISCSI_LOGIN_STAGE_FULL_FEATURE  0x03

#define IS_ISCSI_LOGIN_NSG_OPERATIONAL(x) \
                (GET_ISCSI_FLAG_LOGIN_NSG(x) == ISCSI_LOGIN_STAGE_OPERATIONAL)

#define IS_ISCSI_LOGIN_NSG_FULL_FEATURE(x) \
                (GET_ISCSI_FLAG_LOGIN_NSG(x) == ISCSI_LOGIN_STAGE_FULL_FEATURE)

/* -------------------------------------------------------- 
 * AHSL & DSL
 * -------------------------------------------------------- 
 */

#define ISCSI_AHSL_MASK                 0xFF000000
#define ISCSI_DSL_MASK                  0x00FFFFFF

/* -------------------------------------------------------- 
 * TTT
 * -------------------------------------------------------- 
 */

#define ISCSI_INVALID_ITT               0xFFFFFFFF


/* -------------------------------------------------------- 
 * opcode-specific dword fields, 
 * offsets are indices of: iser_pdu_bhs_t.dword.other[7]
 * -------------------------------------------------------- 
 */

/* SCSI Command */
#define ISCSI_CMD_FIELD_EDTL                0
#define ISCSI_CMD_FIELD_CMD_SN              1
#define ISCSI_CMD_FIELD_EXP_STAT_SN         2

/* Data-OUT */
#define ISCSI_DATA_OUT_FIELD_TTT            0
#define ISCSI_DATA_OUT_FIELD_EXP_STAT_SN    2
#define ISCSI_DATA_OUT_FIELD_DATA_SN        4
#define ISCSI_DATA_OUT_FIELD_OFFSET         5

/* Data-IN */
#define ISCSI_DATA_IN_FIELD_TTT             0
#define ISCSI_DATA_IN_FIELD_STAT_SN         1
#define ISCSI_DATA_IN_FIELD_EXP_CMD_SN      2
#define ISCSI_DATA_IN_FIELD_MAX_CMD_SN      3
#define ISCSI_DATA_IN_FIELD_DATA_SN         4
#define ISCSI_DATA_IN_FIELD_OFFSET          5
#define ISCSI_DATA_IN_FIELD_RESID_CNT       6

/* R2T */
#define ISCSI_R2T_FIELD_TTT                 0
#define ISCSI_R2T_FIELD_STAT_SN             1
#define ISCSI_R2T_FIELD_EXP_CMD_SN          2
#define ISCSI_R2T_FIELD_MAX_CMD_SN          3
#define ISCSI_R2T_FIELD_R2T_SN              4
#define ISCSI_R2T_FIELD_OFFSET              5
#define ISCSI_R2T_FIELD_DESIRED_CNT         6


/* 
 * Printing out PDUs
 * -------------------------------------------------------- 
 */

void iser_pdu_print(char *prefix,	/* to print before the PDU contents */
		    void *p_id,	/* some id printed with prefix as a pointer, conn. id etc. */
		    char *p_bhs,	/* pointer to bhs */
		    struct iser_data_buf_t *p_data);	/* data descriptor */

void print_pdu(char *in_buf, int received_data_size);

struct iscsi_init_scsi_cmnd {	/* Draft 20, Section 10.3 SCSI Command */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd1;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 xfer_len;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u8 cdb[16];
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_scsi_rsp {	/* Draft 20, Section 10.4 SCSI Response */
	__u8 opcode;
	__u8 flags;
	__u8 response;
	__u8 status;
	__u32 length;
	__u64 lun;		/* reserved */
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 exp_data_sn;
	__u32 bidi_resid;
	__u32 resid;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_task_mgt_command {	/* Draft 20, Section 10.5 Task Management
					   Function Request */
	__u8 opcode;
	__u8 function;
	__u16 rsvd1;
	__u32 length;		/* reserved */
	__u64 lun;
	__u32 init_task_tag;
	__u32 ref_task_tag;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u32 ref_cmd_sn;
	__u32 exp_data_sn;
	__u64 rsvd4;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_task_mgt_response {	/* Draft 20, Section 10.6 Task Management
					   Function Response */
	__u8 opcode;
	__u8 flags;
	__u8 response;
	__u8 rsvd1;
	__u32 length;		/* reserved */
	__u64 lun;		/* reserved */
	__u32 init_task_tag;
	__u32 rsvd2;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 rsvd4;
	__u64 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_scsi_data_out {	/* Draft 20, Section 10.7 SCSI Data-out */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd2;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 rsvd3;
	__u32 exp_stat_sn;
	__u32 rsvd4;
	__u32 data_sn;
	__u32 offset;
	__u32 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_scsi_data_in {	/* Draft 20, Section 10.7 SCSI Data-in */
	__u8 opcode;
	__u8 flags;
	__u8 rsvd1;
	__u8 status;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 data_sn;
	__u32 offset;
	__u32 resid;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_r2t {		/* Draft 20, Section 10.8 Ready To Transfer (R2T) */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd2;
	__u32 length;		/* reserved */
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 r2t_sn;
	__u32 offset;
	__u32 xfer_len;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_async_msg {	/* Draft 20, Section 10.9 Asynchronous Message */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd2;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;	/* reserved 0xffffffff */
	__u32 rsvd3;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u8 async_event;
	__u8 async_vcode;
	__u16 parameter1;
	__u16 parameter2;
	__u16 parameter3;
	__u32 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_text_cmnd {	/* Draft 20, Section 10.10 Text Request */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd2;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u64 rsvd4;
	__u64 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_text_rsp {	/* Draft 20, Section 10.11 Text Response */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd2;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 rsvd4;
	__u64 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_login_cmnd {	/* Draft 20, Section 10.12 Login Request */
	__u8 opcode;
	__u8 flags;
	__u8 version_max;
	__u8 version_min;
	__u32 length;
	__u8 isid[6];
	__u16 tsih;
	__u32 init_task_tag;
	__u16 cid;
	__u16 rsvd1;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u64 rsvd2;
	__u64 rsvd3;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_login_rsp {	/* Draft 20, Section 10.13 Login Response */
	__u8 opcode;
	__u8 flags;
	__u8 version_max;
	__u8 version_active;
	__u32 length;
	__u8 isid[6];
	__u16 tsih;
	__u32 init_task_tag;
	__u32 rsvd1;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u8 status_class;
	__u8 status_detail;
	__u16 rsvd2;
	__u64 rsvd3;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_logout_cmnd {	/* Draft 20, Section 10.14 Logout Request */
	__u8 opcode;
	__u8 flags;		/* reasoncode */
	__u16 rsvd1;
	__u32 length;
	__u64 lun;		/* reserved */
	__u32 init_task_tag;
	__u16 cid;
	__u16 rsvd2;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u64 rsvd4;
	__u64 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_logout_rsp {	/* Draft 20, Section 10.15 Logout Response */
	__u8 opcode;
	__u8 flags;
	__u8 response;
	__u8 rsvd1;
	__u32 length;		/* reserved */
	__u64 lun;		/* reserved */
	__u32 init_task_tag;
	__u32 rsvd3;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 rsvd4;
	__u16 time2wait;
	__u16 time2retain;
	__u32 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

/* SNACK pdu structure definition - SAI */

struct iscsi_init_snack {	/* Draft 20, Section 10.16 SNACK Request */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd1;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 rsvd2;
	__u32 exp_stat_sn;
	__u64 rsvd3;
	__u32 begrun;
	__u32 runlen;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_rjt {		/* Draft 20, Section 10.17 Reject */
	__u8 opcode;
	__u8 flags;
	__u8 reason;
	__u8 rsvd2;
	__u32 length;
	__u64 lun;		/* reserved */
	__u32 init_task_tag;	/* reserved 0xffffffff */
	__u32 rsvd4;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 data_sn;
	__u64 rsvd5;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_init_nopout {	/* Draft 20, Section 10.18 NOP-Out */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd1;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 cmd_sn;
	__u32 exp_stat_sn;
	__u64 rsvd2;
	__u64 rsvd3;
	__u32 header_digest;	/* holds header CRC if in use */
};

struct iscsi_targ_nopin {	/* Draft 20, Section 10.19 NOP-In */
	__u8 opcode;
	__u8 flags;
	__u16 rsvd1;
	__u32 length;
	__u64 lun;
	__u32 init_task_tag;
	__u32 target_xfer_tag;
	__u32 stat_sn;
	__u32 exp_cmd_sn;
	__u32 max_cmd_sn;
	__u32 rsvd2;
	__u64 rsvd3;
	__u32 header_digest;	/* holds header CRC if in use */
};


#endif				/* __ISER_BHS_H__ */
