/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_CONN_H__
#define __ISER_CONN_H__

#include "iser.h"

#define ISER_CONN_SET_STATE(x, y)				\
do {								\
	TRACE(TRACE_ISER, "%s: setting conn=%p state to %s\n",	\
	      __func__, (x), #y);				\
	atomic_set(&(x)->state, y);				\
} while (0)

/* iSER API Functions */

iser_status_t
iser_notice_key_values(iser_conn_handle_t conn_h, char *key, char *value);
iser_status_t iser_connection_terminate(iser_conn_handle_t iscsi_conn_h);

iser_status_t
iser_deallocate_connection_resources(iser_conn_handle_t iscsi_conn_h);

/* Extension */
iser_status_t iser_release_control(iser_conn_handle_t iscsi_conn_h,
				   struct iser_recv_pdu_t *p_ctrl_pdu);

/* Internal iSER Functions */

struct iser_conn_t *iser_conn_alloc(char *name);
void iser_conn_get(struct iser_conn_t *);
void iser_conn_put(struct iser_conn_t *);
void iser_conn_destroy(struct iser_conn_t *);

void iser_conn_add_task(struct iser_conn_t *p_iser_conn,
			struct iser_task_t *p_iser_task);

void iser_conn_delete_task(struct iser_conn_t *p_iser_conn,
			   struct iser_task_t *p_iser_task);

void iser_conn_add_dto(struct iser_conn_t *, struct iser_dto_t *);
void iser_conn_remove_dto(struct iser_conn_t *, struct iser_dto_t *);
void iser_conn_add_recv_ctrl_dto(struct iser_conn_t *p_iser_conn,
				 struct iser_dto_t *p_ctrl_dto);

void iser_conn_delete_recv_ctrl_dto(struct iser_conn_t *p_iser_conn,
				    struct iser_dto_t *p_ctrl_dto);

int iser_complete_connection_termination(struct iser_conn_t *p_iser_conn);

//#ifndef NO_BUFF_POOL
//int iser_post_receive_control(struct iser_conn_t *p_iser_conn);
//#else
int iser_post_receive_control(unsigned int size,
			      struct iser_conn_t *p_iser_conn);
//#endif
char *iser_conn_get_state_name(struct iser_conn_t *p_iser_conn);

int iser_conn_async_terminate(struct iser_conn_t *p_iser_conn);
int iser_conn_sync_terminate(struct iser_conn_t *p_iser_conn);

struct iser_conn_t *
iser_adaptor_new_conn(struct iser_adaptor_t  *p_adaptor,
		      char *name);

/**
 * \brief Returns true if there is no more allowable outstanding RDMA
 *        read requests for the given connection.
 */
static inline int iser_conn_read_depth_met(struct iser_conn_t *p_conn)
{
	return atomic_read(&p_conn->outstanding_reads)
	    == p_conn->iser_read_depth;
}

/**
 * Increment the connection's post receive buffer count.
 */
static inline void iser_conn_inc_post_recv_cnt(struct iser_conn_t *p_conn)
{
	atomic_inc(&p_conn->post_recv_buf_count);
}

/**
 * Decrement the connection's post receive buffer count.
 */
static inline void iser_conn_dec_post_recv_cnt(struct iser_conn_t *p_conn)
{
#if defined(CONFIG_ISCSI_DEBUG)
	unsigned int count;
#endif /* CONFIG_ISCSI_DEBUG */

	atomic_dec(&p_conn->post_recv_buf_count);

#if defined(CONFIG_ISCSI_DEBUG)
	count = atomic_read(&p_conn->post_recv_buf_count);
	if (count < p_conn->min_post_recv_bufs) {
		p_conn->min_post_recv_bufs = count;

		TRACE(TRACE_ISER_POSTED_RECVS, "%s: new post recv "
		      "buffer minimum: %u, iSER conn=%p\n",
		      __func__, count, p_conn);
	}
#endif /* CONFIG_ISCSI_DEBUG */
}

#endif				/* __ISER_CONN_H__ */
