/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_DTO_H__
#define __ISER_DTO_H__

#define USE_OFFSET(offset)  (offset)
#define USE_NO_OFFSET	    0

#define USE_SIZE(size)	    (size)
#define USE_ENTIRE_SIZE	    0

int
iser_dto_add_regd_buff(struct iser_dto_t *p_dto,
		       struct iser_regd_buf_t *p_regd_buf,
		       unsigned long use_offset, unsigned long use_size);

void
iser_dto_clone_regd_buffs(struct iser_dto_t *p_dst_dto,
			  struct iser_dto_t *p_src_dto,
			  unsigned long offset, unsigned long size);
int dbg_iser_dto_free(struct iser_dto_t *, char *);
#define iser_dto_free(x) dbg_iser_dto_free(x,(char *)__func__)

int iser_dto_completion_error(struct iser_dto_t *p_dto);

int
iser_dto_add_local_memory(struct iser_dto_t *p_dto,
			  struct iser_data_buf_t *p_mem,
			  struct iser_mem_t *p_phys_vec,
			  unsigned int phys_vec_len,
			  unsigned long *p_total_sz);

void
iser_dto_create_recv_pdu(struct iser_dto_t *p_dto,
			 unsigned long dto_xfer_len);

struct iser_dto_t *iser_dto_send_create(struct iser_conn_t *p_iser_conn,
					union iser_pdu_bhs_t *p_bhs);

struct iser_dto_t *iser_dto_raw_create(struct iser_conn_t *p_conn,
				       unsigned char *p_hello,
				       unsigned int length);

int
iser_dto_copy_send_data(struct iser_dto_t *p_send_dto,
			struct iser_data_buf_t *p_data, unsigned int);

void iser_dto_get(struct iser_dto_t *);
void iser_dto_put(struct iser_dto_t *);
struct iser_dto_t *iser_alloc_dto(struct iser_conn_t *p_conn,
                                  enum iser_dto_type type);

void iser_dto_one_wc(struct iser_dto_t *, unsigned int, unsigned long);
int iser_merge_sges(struct iser_dto_t *);

/**
 * Get a printable string that represents the DTO's type.
 */
static inline const char *dto_type_str(struct iser_dto_t *p_dto)
{
	return p_dto->type == ISER_DTO_RCV ? "ISER_DTO_RCV"
	       : p_dto->type == ISER_DTO_SEND ? "ISER_DTO_SEND"
	       : p_dto->type == ISER_DTO_RDMA_W ? "ISER_DTO_RDMA_W"
	       : p_dto->type == ISER_DTO_RDMA_R ? "ISER_DTO_RDMA_R"
	       : p_dto->type == ISER_DTO_PASSIVE ? "ISER_DTO_PASSIVE"
	       : "<UNKNOWN TYPE>";
}

#endif				// __ISER_DTO_H__
