/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_INITIATOR_H__
#define __ISER_INITIATOR_H__

#include "iser.h"

/* Initiator-speific Functions */

int
iser_initiator_register_entity(struct iser_entity_t *p_entity,
			       struct iser_api_t *api,
			       struct iser_ext_api_t *ext_api,
			       struct iser_ext_api_cb_t *ext_api_cb);

/* Initiator-specific definitions */

/* Maximal bounds on asynchronous PDUs received by Initiator */
#define ISER_INITIATOR_MAX_RX_MISC_PDUS     (ISER_MAX_NOP_IN + ISER_MAX_ASYNC_EVT)
#define ISER_INITIATOR_MAX_TX_MISC_PDUS     (ISER_MAX_TEXT_REQ + ISER_MAX_NOP_OUT + ISER_MAX_TASK_MGT_REQ + ISER_MAX_LOGOUT_REQ)
#define ISER_INITIATOR_MAX_RX_CMD_RESP      ISER_MAX_OUTSTAND_CMDS
#define ISER_INITIATOR_MAX_TX_CMDS          (ISER_MAX_OUTSTAND_CMDS + ISER_MAX_IMMEDIATE_CMDS)

/* EP settings */
#define ISER_INITIATOR_AVG_TASK_RELATED_SEND(first_burst, recv_dsl, imm, max_cmds) \
                                            (max_cmds * (ISER_MAX_CTRLS_PER_CMD(first_burst,recv_dsl,imm) + ISER_MAX_IMMEDIATE_CMDS))

#define ISER_INITIATOR_INITIAL_POST_RECV    ISER_INITIATOR_MAX_RX_MISC_PDUS
#define ISER_INITIATOR_EP_AVG_POST_RECV     (ISER_INITIATOR_MAX_RX_CMD_RESP + ISER_INITIATOR_MAX_RX_MISC_PDUS + ISER_INITIATOR_MAX_TX_MISC_PDUS)
#define ISER_INITIATOR_EP_MAX_RECV_DTOS     (ISER_INITIATOR_EP_AVG_POST_RECV + 8)
#define ISER_INITIATOR_EP_MAX_REQ_DTOS      (ISER_INITIATOR_AVG_TASK_RELATED_SEND(ISER_MAX_FIRST_BURST,ISER_MIN_RECV_DSL,1,ISER_MAX_OUTSTAND_CMDS) + \
                                            + ISER_INITIATOR_MAX_TX_MISC_PDUS + ISER_INITIATOR_MAX_RX_MISC_PDUS)

/* global EVD settings - per entity */
#define ISER_INITIATOR_EVD_MAX_RECV_DTOS    (ISER_INITIATOR_EP_MAX_RECV_DTOS * ISER_INITIATOR_MAX_CONN)
#define ISER_INITIATOR_EVD_MAX_REQ_DTOS     (ISER_INITIATOR_EP_MAX_REQ_DTOS * ISER_INITIATOR_MAX_CONN)
#define ISER_INITIATOR_MAX_QLEN             (ISER_INITIATOR_EVD_MAX_RECV_DTOS + ISER_INITIATOR_EVD_MAX_REQ_DTOS)

#endif				/* __ISER_INITIATOR_H__ */
