/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */


/*
#ifndef MODULE
    #define MODULE
#endif

#ifndef __KERNEL__
    #define __KERNEL__
#endif
*/

#if defined(__KERNEL__)
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/interrupt.h>
#endif				/* __KERNEL__ */

#include "iser.h"
#include "iser_global.h"
#include "iser_trace.h"

/* ---------------------------------------------------------------------
 * Variables
 * ------------------------------------------------------------------ */
int iser_fmr_enabled = 1;
int iser_trace_enabled = 0;
int iser_buffer_aligment_shift = 12;
/* ---------------------------------------------------------------------
 * MODULE REGISTRATION
 * ------------------------------------------------------------------ */
#if defined(__KERNEL__)
MODULE_DESCRIPTION("iSER Datamover");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Alex Nezhinsky,Yonathan Cohen");

module_param(iser_fmr_enabled, int, S_IRWXU);
MODULE_PARM_DESC(iser_fmr_enabled,
		 "enable/disable FMR use (default:enabled)");

module_param(iser_trace_enabled, int, S_IRWXU);
MODULE_PARM_DESC(iser_trace_enabled,
		 "enable/disable trace upon loading (default:disabled)");


module_param(iser_buffer_aligment_shift, int, S_IRWXU);
MODULE_PARM_DESC(iser_buffer_aligment_shift,
		 "shift size for rdma mem registration aligment test (default:4k)");
#endif				/* __KERNEL__ */

/* ---------------------------------------------------------------------
 * MODULE INIT-CLEANUP FUNCTIONS
 * ------------------------------------------------------------------ */

/**
 * init_module - module initialialization function
 */
int iser_init_module(void)
{
	ITRACE(ISER_TRACE_MODULE, "Starting iSER datamover...\n");

	// Initialize the global iSER context
	if (iser_global_init() != 0) {
		IERROR("initializing iser global structures failed!\n");
		return -1;
	}

	IINFO("iSER datamover init done.\n");
	return 0;
}				/* init_module */

/**
 * cleanup_module - module cleanup function
 */
void iser_cleanup_module(void)
{
	ITRACE(ISER_TRACE_MODULE, "Removing iSER datamover...\n");

	iser_global_release();

	IINFO("iSER datamover cleanup done.\n");
}				/* cleanup_module */
