/**
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * \file datamover/iser_ofa.h
 * \brief Contains declarations for the iSER/OFA interfacing functions.
 * \author Ethan Burns
 * \date 2007-04-23
 */
#ifndef _ISER_OFA_H_
#define _ISER_OFA_H_

#ifndef _POSIX_C_SOURCE
#define _POSIX_C_SOURCE 200112L
#endif				/* !_POSIX_C_SOURCE */

#include "iser.h"

#if defined(__KERNEL__)
#include "iser_ofa_kernel.h"
#else			/* __KERNEL__ */
#include "iser_ofa_user.h"
#endif			/* __KERNEL__ */

#define DAT_MEM_PRIV_ALL_FLAG (IBV_ACCESS_LOCAL_WRITE		\
                              | IBV_ACCESS_REMOTE_WRITE		\
                              | IBV_ACCESS_REMOTE_READ		\
                              | IBV_ACCESS_REMOTE_ATOMIC	\
                              | IBV_ACCESS_MW_BIND)

/* ---------------------------------------------------------------------
 * GLOBAL CONTEXT OBJECTS (IA,PZ,EVD,PSP)
 * ------------------------------------------------------------------ */
int iser_ofa_init_adaptor(struct iser_adaptor_t *p_adaptor);
int iser_ofa_deinit_adaptor(struct iser_adaptor_t *p_adaptor);

int iser_listen(struct iser_entity_t *p_entity, struct sockaddr *addr,
		size_t addr_len);
int iser_free_psp_by_entity(struct iser_entity_t *p_entity);
int iser_free_psp_by_adaptor(struct iser_adaptor_t *p_adaptor);

/* ---------------------------------------------------------------------
 * CONNECTION-SCOPE OBJECTS
 * ------------------------------------------------------------------ */
int iser_create_cq_and_qp(struct iser_conn_t *p_iser_conn,
			  unsigned int max_rdma_read_req_in,
			  unsigned int max_rdma_read_req_out);

int iser_destroy_cq_and_qp(struct iser_conn_t *iser_conn);

int iser_connect(struct iser_conn_t *p_conn,
		 struct sockaddr_in *dst_addr,
		 struct sockaddr_in *src_addr);

int iser_disconnect(struct iser_conn_t *p_iser_conn);

int iser_accept(struct iser_conn_t *p_iser_conn,
		struct iser_data_buf_t *p_priv_data);

int iser_reject(struct iser_conn_t *p_iser_conn);

/* ---------------------------------------------------------------------
 * THREAD MAIN LOOPS
 * ------------------------------------------------------------------ */
int iser_spawn_event_thread(struct iser_adaptor_t *p_adaptor);
void iser_kill_event_thread(struct iser_adaptor_t *p_adaptor);
int iser_do_poll_cq(struct iser_conn_t *p_conn);
void iser_suicide_thread(void);
thread_ret iser_event_handler_thread(void *arg);

/* ---------------------------------------------------------------------
 * MEMORY REGISTRATION
 * ------------------------------------------------------------------ */
int
iser_register_phys_mem(struct iser_conn_t *p_conn,
		       struct iser_mem_t *p_phys_vec,
		       unsigned int phys_vec_len,
		       unsigned long total_sz,
		       unsigned int priv_flags,
		       struct iser_mem_handles_t *p_mem_reg,
		       uint32_t * p_rmr_ctxt,
                       enum dma_data_direction dir);

int iser_unregister_memory(struct iser_mem_handles_t *mem_reg);

/* ---------------------------------------------------------------------
 * POSTING RECEIVE BUFFERS
 * ------------------------------------------------------------------ */
int iser_post_recv(struct iser_dto_t *p_recv_dto);

/* ---------------------------------------------------------------------
 * DTO
 * ------------------------------------------------------------------ */
int iser_start_dto(struct iser_dto_t *p_dto);

void iser_free_osc_buf(struct iser_dto_t *p_dto);
int iser_alloc_osc_buf(struct iser_dto_t *p_dto);

#endif				/* !_ISER_OFA_H_ */
/* vi: set tabstop=8 textwidth=78: */
