/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_TRACE_H__
#define __ISER_TRACE_H__

/**
 * \brief True if iSER tracing is on.
 */
#define ITRACEABLE(x) (TRACE_TEST(TRACE_ISER))

/**
 * \brief An iSER error message.
 */
#define IERROR(x...) TRACE_ERROR(x)

/**
 * \brief an iSER panic message.
 */
#if !defined(__KERNEL__)
#define IPANIC(x...)		\
	do {			\
		TRACE_ERROR(x);	\
		abort();	\
	} while (0);
#else
#define IPANIC(x...) panic(x)
#endif /* !__KERNEL__ */

/**
 * \brief An iSER trace message.
 */
#define ITRACE(iser_mask_ignored, x...) TRACE(TRACE_ISER, x)

/**
 * \brief An informative message.
 */
#define IINFO(x...) TRACE(TRACE_ISER, x)

/**
 * \brief Trace a function entry.
 */
#define ITRACE_ENTRY() TRACE(TRACE_ENTER_LEAVE, "++ Entering %s\n", __func__)

/**
 * \brief Trace a function exit.
 */
#define ITRACE_EXIT() TRACE(TRACE_ENTER_LEAVE, "-- Leaving %s\n", __func__)

#endif				/* !__ISER_TRACE_H__ */
