/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2007-2008 UNH InterOperability Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 *
 * The name IOL and/or UNH may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 */

#ifndef __ISER_UTILS_H__
#define __ISER_UTILS_H__

#include "iser.h"

/* ---------------------------------------------------------------------
 * GENERIC HASH MANAGEMENT
 * ------------------------------------------------------------------ */

void hash_init(struct hash_table_t *hash_table);

/* ---------------------------------------------------------------------
 * ISER TASK-SPECIFIC HASH MANAGEMENT
 * ------------------------------------------------------------------ */

struct iser_task_t *hash_find_iser_task(struct iser_conn_t *iser_conn,
					u32 itt);

void hash_add_iser_task(struct iser_task_t *iser_task);

void hash_delete_iser_task(struct iser_task_t *iser_task);

/* ---------------------------------------------------------------------
 * ISER CONNECTION-SPECIFIC HASH MANAGEMENT
 * ------------------------------------------------------------------ */

struct iser_conn_t *hash_find_iser_conn(iser_conn_handle_t iscsi_conn_h);

void hash_add_iser_conn(struct iser_conn_t *iser_conn);

void hash_delete_iser_conn(struct iser_conn_t *iser_conn);

/* ---------------------------------------------------------------------
 * BUFFER DESCRIPTORS 
 * ------------------------------------------------------------------ */

unsigned long iser_get_data_total_length(struct iser_data_buf_t *p_data);

int
iser_iovec_virt_to_mem_t(struct iser_task_t *p_task,
			 enum dma_data_direction dir,
			 struct iser_data_buf_t *p_data,
			 struct iser_mem_t *p_phys,
			 unsigned long *p_total_sz);

int
iser_scatterlist_to_mem_t(struct iser_task_t *p_task,
			  enum dma_data_direction cmd_dir,
			  struct iser_data_buf_t *p_data,
			  struct iser_mem_t *p_phys,
			  unsigned long *p_total_sz);

unsigned int
iser_iovec_aligned_length(struct iser_mem_t *p_phys,
			  unsigned int phys_vec_sz,
			  unsigned long *total_data_sz);

void *iser_phys_to_virt(void *phys_addr);

#if 0
void *iser_page_to_virt(struct page *page);
#endif

void *iser_iovec_virt_entry_addr(void *p_iovec_virt, int i);

void *iser_scatterlist_entry_to_virt(void *p_scatterlist, int i);

unsigned long iser_iovec_entry_len(void *p_iovec_arr, int i);

unsigned long iser_scatterlist_entry_len(void *p_scatterlist, int i);

void
iser_data_buf_memcpy(struct iser_data_buf_t *p_dst_buf,
		     struct iser_data_buf_t *p_src_data,
		     unsigned long *p_total_copied_sz);

char *iser_data_buf_get_type_name(struct iser_data_buf_t *p_data);

#endif				/* __ISER_UTILS_H__ */
