
/*
 * Copyright (c) 2004, 2005, 2006 Voltaire, Inc. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef __ISER_KDAPL_H__
#define __ISER_KDAPL_H__

#include "iser.h"

/* ---------------------------------------------------------------------
 * GLOBAL CONTEXT OBJECTS (IA,PZ,EVD,PSP)
 * ------------------------------------------------------------------ */
int iser_create_ia_pz_evd(struct iser_adaptor_t *p_adaptor);
int iser_free_ia_pz_evd(struct iser_adaptor_t *p_adaptor);

int iser_create_psp(struct iser_entity_t *p_entity, unsigned int port);
int iser_free_psp_by_entity(struct iser_entity_t *p_entity);
int iser_free_psp_by_adaptor(struct iser_adaptor_t *p_adaptor);

/* ---------------------------------------------------------------------
 * CONNECTION-SCOPE OBJECTS
 * ------------------------------------------------------------------ */
int iser_create_ep(struct iser_conn_t *p_iser_conn,
		   unsigned long max_rdma_sz,
		   unsigned int max_rdma_read_in,
		   unsigned int max_rdma_read_out,
		   unsigned int max_recv_dtos,
		   unsigned int max_request_dtos,
		   unsigned int max_request_iov);

int iser_connect(struct iser_conn_t *iser_conn,
		 struct sockaddr_in *dst_addr,
		 struct sockaddr_in *src_addr);

int iser_disconnect(struct iser_conn_t *p_iser_conn);

int iser_free_ep(struct iser_conn_t *iser_conn);

int iser_cr_accept(struct iser_conn_t *p_iser_conn,
		   struct iser_data_buf_t *p_priv_data);

int iser_cr_reject(struct iser_conn_t *p_iser_conn);

/* ---------------------------------------------------------------------
 * THREAD MAIN LOOPS
 * ------------------------------------------------------------------ */
int iser_spawn_event_thread(struct iser_adaptor_t *p_adaptor);
void iser_kill_event_thread(struct iser_adaptor_t *p_adaptor);
void iser_suicide_thread(void);
int iser_event_handler_thread(void *arg_unused);


/* ---------------------------------------------------------------------
 * MEMORY REGISTRATION
 * ------------------------------------------------------------------ */
int iser_register_virt_mem(struct iser_adaptor_t *p_adaptor,
			   void *p_buf,
			   unsigned long data_sz,
			   DAT_MEM_PRIV_FLAGS priv_flags,
			   struct iser_mem_handles_t *mem_reg,
			   DAT_RMR_CONTEXT * rmr_ctxt);

int
iser_register_phys_mem(struct iser_adaptor_t *p_adaptor,
		       struct iser_phys_mem_t *p_phys_vec,
		       unsigned int phys_vec_len,
		       unsigned long total_sz,
		       DAT_MEM_PRIV_FLAGS priv_flags,
		       struct iser_mem_handles_t *p_mem_reg,
		       DAT_RMR_CONTEXT * p_rmr_ctxt,
		       enum iser_entity_type entity_type);

#ifdef _DAT1_2_
int
iser_register_ia_mem(struct iser_adaptor_t *p_adaptor,
		     void *virt_addr,
		     unsigned long data_sz,
		     DAT_MEM_PRIV_FLAGS priv_flags,
		     struct iser_mem_handles_t *p_mem_reg,
		     DAT_RMR_CONTEXT * p_rmr_ctxt);

#endif

int iser_unregister_memory(struct iser_mem_handles_t *mem_reg);

/* ---------------------------------------------------------------------
 * POSTING RECEIVE BUFFERS
 * ------------------------------------------------------------------ */
int iser_post_recv(struct iser_dto_t *p_recv_dto);

/* ---------------------------------------------------------------------
 * DTO
 * ------------------------------------------------------------------ */
int iser_start_dto(struct iser_dto_t *p_dto);

#endif				//__ISER_KDAPL_H__
