/*	initiator/iscsi_initiator.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	This file contains all the required header files, struct
 *      definitions and constants for iscsi initiator code.
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 * 	written permission.
*/
#ifndef	_ISCSI_INITIATOR_H
#define	_ISCSI_INITIATOR_H

#include "../common/queue.h"

#ifndef __KERNEL__
#define scsi_buffer(cmnd) (cmnd)->request_buffer
#define scsi_bufflen(cmnd) (cmnd)->request_bufflen
#define scsi_sg_count(cmnd) (cmnd)->use_sg
#endif /* !__KERNEL__ */

#ifdef __KERNEL__

/* RDR */
#ifndef K26
#include <linux/blk.h>
#endif

#include <linux/version.h>
#include <linux/blkdev.h>
#include <linux/kernel.h>
#include <linux/ctype.h>
#include <linux/string.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/unistd.h>
#include <linux/smp_lock.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <net/sock.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <scsi.h>

/* Ming Zhang, mingz@ele.uri.edu */
#ifdef K26
#include <scsi/scsi_device.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi.h>

/* 2.6.12 no longer defines these symbols in drivers/scsi/scsi.h
 * instead, the sc_data_direction field in struct scsi_cmnd in scsi/scsi_cmnd.h
 * is now defined to have type enum dma_data_direction, which is defined
 * in linux/dma_mapping.h, and that is included by scsi/scsi_cmnd.h
 * (in 2.4, struct scsi_cmnd was defined in drivers/scsi/scsi.h, and the
 * sc_data_direction field was an unsigned char)
 * (in 2.6.6, the following defines were still in drivers/scsi/scsi.h)
 */
#ifndef SCSI_DATA_UNKNOWN
#define SCSI_DATA_UNKNOWN	(DMA_BIDIRECTIONAL)
#endif
#ifndef SCSI_DATA_WRITE
#define SCSI_DATA_WRITE		(DMA_TO_DEVICE)
#endif
#ifndef SCSI_DATA_READ
#define SCSI_DATA_READ		(DMA_FROM_DEVICE)
#endif
#ifndef SCSI_DATA_NONE
#define SCSI_DATA_NONE		(DMA_NONE)
#endif

#else
#include <sd.h>
#endif

#include "../common/scatterlist.h"
#include "../common/scsi_cmnd.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
#include <hosts.h>
#else
#include <scsi/scsi_host.h>
#endif

#include <linux/module.h>

#else

#include "../common/user_space.h"
#include "../common/user_scsi.h"

/* needed to get the definition of "struct timeval" */
#include <sys/time.h>

/* skeleton from /usr/src/linux/include/scsi/scsi_cmnd.h */
struct scsi_cmnd {

	struct scsi_device *device;

	unsigned char cmd_len;		/* no. of bytes in CDB */
	unsigned char old_cmd_len;
	enum dma_data_direction sc_data_direction;
	enum dma_data_direction sc_old_data_direction;

	unsigned char cmnd[MAX_COMMAND_SIZE];	/* the CDB */
	unsigned request_bufflen;	/* Actual request size */
	void *request_buffer;		/* Actual requested buffer */

	unsigned short old_use_sg;	/* save use_sg here during sense info */
	unsigned short use_sg;		/* Number of pieces of scatter-gather */

#define SCSI_SENSE_BUFFERSIZE   96
	unsigned char sense_buffer[SCSI_SENSE_BUFFERSIZE];
	/* obtained by REQUEST SENSE when
	 * CHECK CONDITION is received on original
	 * command (auto-sense) */

	/* Low-level done function - can be used by low-level driver to point
	 *        to completion function.  Not used by mid/upper level code. */
	void (*scsi_done) (struct scsi_cmnd *);

	int result;		/* Status code from lower level driver */
};


/* copies from /usr/src/linux/include/linux/notifier.h */

#define	NOTIFY_DONE	 0x0000	/* don't care */
#define NOTIFY_OK	 0x0001	/* Suits me */
#define	NOTIFY_STOP_MASK 0x8000	/* Don't call further */
#define	NOTIFY_BAD	 (NOTIFY_STOP_MASK|0x0002)	/* Bad/Veto action */

#define	SYS_DOWN	0x0001	/* Notify of system down */
#define	SYS_RESTART	SYS_DOWN
#define	SYS_HALT	0x0002	/* Notify of system halt */
#define SYS_POWER_OFF	0x0003	/* Notify of system power off */

struct notifier_block {
	int (*notifier_call) (struct notifier_block *, unsigned long,
			      void *);
	struct notifier_block *next;
	int priority;
};

struct Scsi_Host;


/* skeleton from /usr/src/linux/include/scsi/scsi_host.h */
struct scsi_host_template {
	const char *name;

	/*
	 * Used to initialize old-style drivers.  For new-style drivers
	 * just perform all work in your module initialization function.
	 *
	 * Status:  OBSOLETE
	 */
	int (*detect) (struct scsi_host_template *);

	/*
	 * Used as unload callback for hosts with old-style drivers.
	 *
	 * Status: OBSOLETE
	 */
	int (*release) (struct Scsi_Host *);

	/*
	 * The queuecommand function is used to queue up a scsi
	 * command block to the LLDD.  When the driver finished
	 * processing the command the done callback is invoked.
	 *
	 * If queuecommand returns 0, then the HBA has accepted the
	 * command.  The done() function must be called on the command
	 * when the driver has finished with it. (you may call done on the
	 * command before queuecommand returns, but in this case you
	 * *must* return 0 from queuecommand).
	 *
	 * Queuecommand may also reject the command, in which case it may
	 * not touch the command and must not call done() for it.
	 *
	 * There are two possible rejection returns:
	 *
	 *   SCSI_MLQUEUE_DEVICE_BUSY: Block this device temporarily, but
	 *   allow commands to other devices serviced by this host.
	 *
	 *   SCSI_MLQUEUE_HOST_BUSY: Block all devices served by this
	 *   host temporarily.
	 *
	 * For compatibility, any other non-zero return is treated the
	 * same as SCSI_MLQUEUE_HOST_BUSY.
	 *
	 * NOTE: "temporarily" means either until the next command for#
	 * this device/host completes, or a period of time determined by
	 * I/O pressure in the system if there are no other outstanding
	 * commands.
	 *
	 * STATUS: REQUIRED
	 */
	int (*queuecommand) (struct scsi_cmnd *,
			     void (*done) (struct scsi_cmnd *));

	/*
	 * This is an error handling strategy routine.  You don't need to
	 * define one of these if you don't want to - there is a default
	 * routine that is present that should work in most cases.  For those
	 * driver authors that have the inclination and ability to write their
	 * own strategy routine, this is where it is specified.  Note - the
	 * strategy routine is *ALWAYS* run in the context of the kernel eh
	 * thread.  Thus you are guaranteed to *NOT* be in an interrupt
	 * handler when you execute this, and you are also guaranteed to
	 * *NOT* have any other commands being queued while you are in the
	 * strategy routine. When you return from this function, operations
	 * return to normal.
	 *
	 * See scsi_error.c scsi_unjam_host for additional comments about
	 * what this function should and should not be attempting to do.
	 *
	 * Status: REQUIRED     (at least one of them)
	 */
	int (*eh_abort_handler) (struct scsi_cmnd *);
	int (*eh_device_reset_handler) (struct scsi_cmnd *);
	int (*eh_bus_reset_handler) (struct scsi_cmnd *);
	int (*eh_host_reset_handler) (struct scsi_cmnd *);

	/*
	 * Name of proc directory
	 */
	char *proc_name;

	/*
	 * This determines if we will use a non-interrupt driven
	 * or an interrupt driven scheme,  It is set to the maximum number
	 * of simultaneous commands a given host adapter will accept.
	 */
	int can_queue;

	/*
	 * In many instances, especially where disconnect / reconnect are
	 * supported, our host also has an ID on the SCSI bus.  If this is
	 * the case, then it must be reserved.  Please set this_id to -1 if
	 * your setup is in single initiator mode, and the host lacks an
	 * ID.
	 */
	int this_id;

	/*
	 * This determines the degree to which the host adapter is capable
	 * of scatter-gather.
	 */
	unsigned short sg_tablesize;

	/*
	 * True if this host adapter can make good use of linked commands.
	 * This will allow more than one command to be queued to a given
	 * unit on a given host.  Set this to the maximum number of command
	 * blocks to be provided for each device.  Set this to 1 for one
	 * command block per lun, 2 for two, etc.  Do not set this to 0.
	 * You should make sure that the host adapter will do the right thing
	 * before you try setting this above 1.
	 */
	short cmd_per_lun;

	/*
	 * List of hosts per template.
	 *
	 * This is only for use by scsi_module.c for legacy templates.
	 * For these access to it is synchronized implicitly by
	 * module_init/module_exit.
	 */
	struct list_head legacy_hosts;

};

/* skeleton from /usr/src/linux/include/scsi/scsi_host.h */
struct Scsi_Host {
	struct scsi_host_template *hostt;

	unsigned short host_no;

	/*
	 * These three parameters can be used to allow for wide scsi,
	 * and for host adapters that support multiple busses
	 * The first two should be set to 1 more than the actual max id
	 * or lun (i.e. 8 for normal systems).
	 */
	unsigned int max_id;
	unsigned int max_lun;
	unsigned int max_channel;

	/*
	 * This is a unique identifier that must be assigned so that we
	 * have some way of identifying each detected host adapter properly
	 * and uniquely.  For hosts that do not support more than one card
	 * in the system at one time, this does not need to be set.  It is
	 * initialized to 0 in scsi_register.
	 */
	unsigned int unique_id;

	/*
	 * The maximum length of SCSI commands that this host can accept.
	 * Probably 12 for most host adapters, but could be 16 for others.
	 * For drivers that don't set this field, a value of 12 is
	 * assumed.  I am leaving this as a number rather than a bit
	 * because you never know what subsequent SCSI standards might do
	 * (i.e. could there be a 20 byte or a 24-byte command a few years
	 * down the road?).  
	 */
	unsigned char max_cmd_len;

	int this_id;
	int can_queue;
	short cmd_per_lun;
	short unsigned int sg_tablesize;
	short unsigned int max_sectors;

	/*
	 * List of hosts per template.
	 *
	 * This is only for use by scsi_module.c for legacy templates.
	 * For these access to it is synchronized implicitly by
	 * module_init/module_exit.
	 */
	struct list_head sht_legacy_list;

	/* rdr - added this here because it has to go somewhere
	 *	 and if it is here we can access it through global_host
	 * probably should be a list threaded through "siblings" field in
	 * struct scsi_device, because the comment on that field says
	 * "list of all devices on this host"
	 * we can get away with not having a list as long as there is only
	 * one device on this host
	 */
	struct scsi_device	*device;

	/*
	 * We should ensure that this is aligned, both for better performance
	 * and also because some compilers (m68k) don't automatically force
	 * alignment to a long boundary.
	 */
	unsigned long hostdata[0]	/* Used for storage of host specific stuff */
	    __attribute__ ((aligned(sizeof(unsigned long))));

};

/* copies from /usr/src/linux/include/scsi/scsi_host.h */
/* legacy interfaces */
extern struct Scsi_Host *scsi_register(struct scsi_host_template *, int);
extern void scsi_unregister(struct Scsi_Host *);

#endif

#include "../common/iscsi_common.h"
#include "../common/debug.h"
#include "../common/my_memory.h"
#include "../common/range.h"
#include "../common/crc.h"
#include "../common/tcp_utilities.h"

/* chap and srp support - CHONG */
#include "../security/misc/misc_func.h"
#include "../security/chap/chap.h"
#include "../security/srp/srp.h"

#include "../common/text_param.h"
#include "../common/initiator_negotiate.h"

#ifdef RDMA_MODE
#include "../datamover/include/iser_api.h"
#endif
#include "iser_initiator.h"


/*	number of seconds to wait for a task management response to an abort */
#define ABORT_TIMEOUT		(5 * HZ)

/*	maximum number of iov slots iscsi_initiator can set up for I/O	*/
#define MAX_IOV_SLOTS		1028

/*	maximum number of commands scsi midlevel can queue for 1 lun */
#define MAX_COMMANDS_PER_LUN	18

/*	maximum number of targets this iscsi_initiator host will register for */
#define MAX_TARGET_IDS		10

/*	maximum number of luns each target can handle */
#define MAX_LUNS		8

/*	maximum number of bytes we will accept in a scsi cdb */
#define MAX_CDB_LEN		16

/*  Multiple connections Round Robin Scheduling Scheme - SAI */
#define CONN_SCHED_RR		1
#define CONN_SCHED_LUN		2

/* Add Maximum Connections allowed per session - SAI */
#define MAX_CONNECTIONS		20

/* Default (initial) settings for initiator's 6-byte ISID
 * RFC 3720 Section 10.12.5 ISID
 */
#define DEFAULT_ISID_TYPE	2	/* random */
#define DEFAULT_ISID_NUMBER	0x123456	/* 24-bit B&C */
#define DEFAULT_ISID_QUALIFIER	0x789a	/* 16-bit D */

/* Size of chunks allocated to connection's accepted_stat_sn[] and isvalid[] */
#define ACCEPTED_CHUNK		32

/*	Holds pointer to private data part of Scsi_Host structure.
 *	This is set up once, by iscsi_detect()
 *	which is called when the initiator module is loaded.
 */
extern struct iscsi_hostdata *global_hostdata;

/*	Holds pointer to the registered Scsi_Host. */
extern struct Scsi_Host *global_host;


#ifdef UNH_LOCK_DEBUG

/* generate debugging code for UNH locks */

#if !defined(CONFIG_SMP)

/* generate debugging code for UNH locks on uniprocessors */

typedef struct unh_lock_type {
	int lock;
	int line;
	void *flags;
} unh_lock_t;
#define UNH_LOCK_TYPE	unh_lock_t

#define UNH_LOCK_UNLOCKED	(unh_lock_t) { 0, 0, 0 }

#define UNH_LOCK(l, f)	do { \
			    local_irq_save(f); \
			    if ((l)->lock != 0) { \
				printk("%s:%d:   LOCK value %d should be 0!\n",\
					__FUNCTION__, __LINE__, (l)->lock); \
			    	printk("last LOCK line %d\n", (l)->line); \
			    } \
			    if ((l)->flags) { \
				printk("%s:%d: non-zero flags %p\n", \
				__FUNCTION__, __LINE__, (l)->flags); \
			    	printk("last LOCK line %d\n", (l)->line); \
			    } \
			    (l)->lock += 1; \
			    (l)->line = __LINE__; \
			    (l)->flags = &f; \
			} while(0)

#define UNH_UNLOCK(l,f)	do { (l)->lock -= 1; \
			    if ((l)->lock != 0) { \
				printk("%s:%d: UNLOCK value %d should be 0!\n",\
					__FUNCTION__, __LINE__, (l)->lock); \
			    	printk("last LOCK line %d\n", (l)->line); \
			    } \
			    if ((l)->flags != &f) { \
				printk("%s:%d flags %p expected %p\n", \
				__FUNCTION__, __LINE__, &(f), (l)->flags); \
			    	printk("last LOCK line %d\n", (l)->line); \
			    } \
			    (l)->flags = 0; \
			    local_irq_restore(f); \
			} while(0)
#else

/* generate debugging code for UNH locks on multiprocessors */

typedef struct unh_lock_type {
	spinlock_t lock;
	int line;
	void *flags;
} unh_lock_t;
#define UNH_LOCK_TYPE	unh_lock_t

#define UNH_LOCK_UNLOCKED	(unh_lock_t) { {1}, 0, 0 }

#define UNH_LOCK(l,f)   do { if(TRACE_TEST(TRACE_SEM)) { \
				printk("%s:%d: Try %p\n", \
				__FUNCTION__, __LINE__, (l)); \
				spin_lock_irqsave(&(l)->lock,f); \
				printk("%s:%d: Got %p\n", \
				__FUNCTION__, __LINE__, (l)); \
			    } else { \
				spin_lock_irqsave(&(l)->lock,f); \
			    } \
			    if ((l)->flags) { \
				printk("%s:%d: non-zero flags %p\n", \
				__FUNCTION__, __LINE__, (l)->flags); \
			    	printk("%s last LOCK line %d\n", \
				current->comm, (l)->line); \
				spin_unlock_irqrestore (&(l)->lock,f); \
				BUG(); \
			    } \
			    (l)->line = __LINE__; \
			    (l)->flags = &f; \
			} while(0)

#define UNH_UNLOCK(l,f) do { \
			    if(TRACE_TEST(TRACE_SEM)) { \
				printk("%s:%d: Out %p\n", \
				__FUNCTION__, __LINE__, l); \
			    } \
			    if ((l)->flags != &(f)) { \
				printk("%s:%d flags %p expected %p\n", \
				__FUNCTION__, __LINE__, &(f), (l)->flags); \
			    	printk("%s last LOCK line %d\n", \
				current->comm, (l)->line); \
				spin_unlock_irqrestore (&(l)->lock,f); \
				BUG(); \
			    } \
			    (l)->flags = 0; \
			    spin_unlock_irqrestore (&(l)->lock,f); \
			} while(0)
#endif
#else

/* do NOT generate debugging code for UNH locks */

#define UNH_LOCK_TYPE		spinlock_t
#define UNH_LOCK_UNLOCKED	SPIN_LOCK_UNLOCKED

#define UNH_LOCK(l,f)		spin_lock_irqsave(l,f)
#define UNH_UNLOCK(l,f)		spin_unlock_irqrestore (l,f)

#endif


/*	Used for all access to the shared structures global_host and during
 *	the operation of any single thread to prevent things from
 *	disappearing from under the thread.
 */
extern UNH_LOCK_TYPE host_data_lock;

/*	values for the session_state field of the session structure */
enum session_states {

	SESSION_NOT_PRESENT,	/* set in build_session_skeleton()
				   as initial value */
	SESSION_LOGGED_IN,	/* set in iscsi_initiator_login() after a
				   successful return from
				   initiator_parameter_negotiate() */
	SESSION_FULL_FEATURE_PHASE,	/* set in create_session() after a
					   successful return from init_session()
					   where tx and rx threads for first
					   connection in session are confirmed
					   to be up ok */
	SESSION_LOGGED_OUT,	/* set in rx_logout_rsp() after receiving a
				   successful logout response to a request of
				   type LOGOUT_CLOSE_SESSION */
	SESSION_CLOSED		/* set at end of iscsi_initiator_rx_thread()
				   by first rx thread to detect
				   SESSION_LOGGED_OUT, or by the only rx thread
				   in a discovery session */
};

/*	values for the connection_state field of the connection structure */
enum connection_states {

	CONNECTION_NOT_PRESENT,	/* set in build_session_skeleton()
				   as initial value.
				   Also set in close_connection(),
				   and init_connection() after
				   connection is closed */
	CONNECTION_CONNECTED,	/* set in init_connection() after
				   socket has been connected ok */
	CONNECTION_LOGGED_IN,	/* set in iscsi_initiator_login() after
				   successful return from
				   initiator_parameter_negotiate(), so
				   login phase has completed ok */
	CONNECTION_FULL_FEATURE_PHASE,	/* set in init_connection() after rx
					   thread for connection is confirmed
					   to be up ok */
	CONNECTION_RECOVERING,	/* set in deal_with_connrec() after
				   Time2Wait expires */
	CONNECTION_LOGGED_OUT,	/* set in rx_logout_rsp() after a successful
				   logout response has been received to a
				   request of type LOGOUT_CLOSE_CONNECTION */
	CONNECTION_DISCONNECTED,	/* set at end of iscsi_initiator_rx_thread()
					   after rx thread shuts down */
	CONNECTION_TERMINATED	/* added by Minti */
};

/*	values for the cmd_error_state field of the command structure */
enum cmd_error_states {
	CMD_OK_SO_FAR,		/* no errors yet */
	CMD_NEEDS_SNACK,	/* need to send snack when remaining
				   data ins stop arriving */
	CMD_USING_SNACK		/* when receiving data ins in response
				   to a snack */
};

#define WHITE_SPACE " \t\v\f\n\r"

/*	Holds information extracted from a real or implied R2T from target.
 *	Used by initiator to control sending DataOut PDUs to target.
 */
struct r2t_cookie {
	struct list_head link;
	struct list_head order_link;	/* order of this r2t on a connection */
	__u32 target_xfer_tag;	/*opaque -- in network order (from target r2t) */
	__u32 offset;		/* SCSI buffer offset of next InOrder Data Out */
	/* but is limit of next Out-of-Order Data Out */
	__u32 r2t_xfer_length;	/* no. of bytes left to send in r2t sequence  */
	/* or no. received so far in DataIn sequence  */
	__u32 data_out_sn;	/* incremented from 0 by DataOuts for this r2t */
};

struct command {
	struct list_head link;	/* for threaded, doubly-linked circular lists */
	__u8 cmd_state;		/* state of this cdb/cmd */
	__u8 header_opcode;	/* copy of opcode when sending a pdu */
	__u8 header_flags;	/* copy of flags when sending a pdu */
	__u8 activity_flg;	/* Stores Activity from target for command SAI */
	__u8 need_write_built;	/* Create write infrastruct within xmit thread */
	__u8 header_complete;	/* ExpStatSN and header CRC have been done */
	__u32 init_task_tag;
	__u32 lun;		/* LUN if SCpnt (below) is not NULL */
	__u32 data_offset;	/* offset to next data to receive or r2t */
	__u32 recvd_length;	/* bytes received so far in DataIn sequence */
	__u32 still_to_send;	/* total data bytes left to sent in write cmd */
	__u32 data_in_sn;	/* incremented from 0 by DataIns for this cmd */
	__u32 cmd_msg_flags;	/* MSG_MORE on writes when more data follows */
	struct order_range r2t_range_list;	/* for out-of-order r2ts */
	struct order_range pdu_range_list;	/* for out-of-order DataIn pdu */
	__u32 n_r2t_cookies;	/* no. of real (not implied) r2ts outstanding */
	__u32 r2t_sn;		/* incremented from 0 by r2ts */
	void *buffer_to_free;	/* free this buffer after xmit (if not NULL) */
	__u8 snack_flags;	/* Command response and SNACK reject bits SAI */
	__u8 cmd_error_state;	/* CMD_OK_SO_FAR, ... */
	__u8 task_mgt_response;	/* for TM command, the target's response */
	__u8 task_mgt_function;	/* for TM command, the initiator's function */
	struct command *related_cmd;	/* for TM command, the related command */
	__u32 cmd_error_begrun;	/* first missing DataSn to ask for in SNACK */
	__u32 cmd_error_offset;	/* offset of first missing DataIn pdu */
	__u32 cmd_error_runlen;	/* number of missing DataIn pdus to ask for */
	__u32 cmd_error_size;	/* size of good DataIn pdus since last bad one */
	__u32 tx_data_length;	/* length of attached data for tx stats */
	struct iscsi_init_scsi_cmnd iscsi_cmd;	/* SCSI Cmnd PDU hdr blt here */

	/* structures from this point on are NOT zeroed on command setup!
	 * Therefore, do NOT move the position of "SCpnt" unless you change
	 * the memset() in setup_command()!
	 */
	struct scsi_cmnd *SCpnt;	/* pointer to SCSI command */
	struct list_head r2t_cookies;	/* list of unsatisfied r2ts */
	struct timer_list abort_timer;	/* timer for sending TM aborts */
	struct connection *my_conn;	/* connection this command belongs to */
	__u32 abit_flags;	/* should be __u8 - flags for A-bi rx */
	__u32 abit_in_sn;	/* saved data_in_sn for A-bit rexmit */
	__u32 abit_ttt;		/* saved target_xfer_tag for A-bit rx */
	__u64 abit_lun;		/* saved lun for A-bit rexmit */
	__u32 abit_offset;	/* saved offset for A-bit rx */
	__u32 abit_resid;	/* saved residual count for A-bit rx */
	__u32 ref_task_tag;	/* TASK REASSIGN needs this */
	__u32 new_max_recv_length;	/*new MRDSL sent to target in FFP text */
	__u32 new_max_send_length;	/*new MRDSL from target in FFP text */
	unsigned long time_stamp;	/* time when command xmitted */
	unsigned long tx_wait_to_send;	/* deadline til next send of this cmd */
	__u32 stat_sn;		/* StatSN in command response */
	__u32 tx_size;		/* no. of bytes to send in sendmsg */
	__u32 tx_sent_so_far;	/* no. of bytes sent in prev sendmsgs */
	__u32 retransmit_tx_size;	/* original command tx_size - SAI */
	__u32 tx_iovlen;	/* no. of used tx_iov[]slots  */
	struct iovec tx_iov[MAX_IOV_SLOTS];	/* socket iovector built here */
	struct iovec tx_iov_copy[MAX_IOV_SLOTS];	/* copy of tx_iov for sendmsg */
	struct msghdr tx_msghdr;	/* socket message hdr blt here */
	__u32 data_checksum;	/* checksum for attached data */
	struct iscsi_init_scsi_data_out data;	/* DataOut PDU hdr built here */
#ifdef K26
	struct scatterlist sg[MAX_IOV_SLOTS];
	__u32 pindex;
	__u32 padding;
#endif
};

/*
 * cmd_state state flags
 */

#define CMD_STATE_TXDONE	0x01	/* tx_thread is done with cmd */
#define CMD_STATE_RXDONE	0x02	/* rx_thread is done with cmd */
#define CMD_STATE_REXMIT_ORIG	0x04	/* rebuild orig cmd in case of rexmit */
#define CMD_STATE_TXSTARTED	0x08	/* tx_thread sent something on cmd */
#define CMD_STATE_ABORTING	0x10	/* command is being aborted */
#define CMD_STATE_ABORT_FINISH	0x20	/* TM command timed out or responsed */


#define CMD_NEEDS_WRITE_BUILT 0x01	/* xmit thread needs to call build_write */

/*
 * The following structure holds the statistic value,
 * it is a per session structure.
 */
#ifdef ISCSI_STATS

/*
 * These are some session wide stats
 */

struct iscsi_session_stats {
	__u32 iscsi_ssn_noptx_pdus;	/* #of nop PDUs txd */
	__u32 iscsi_ssn_scsicmd_pdus;	/* #of command PDUs transferred */
	__u32 iscsi_ssn_tskmgmtcmd_pdus;	/* #of tast mgmt PDUs */
	__u32 iscsi_ssn_login_pdus;	/* #of login req pdus */
	__u32 iscsi_ssn_text_pdus;	/* #of test req pdus */
	__u32 iscsi_ssn_dataout_pdus;	/* #of data out -WRITE PDUs */
	__u32 iscsi_ssn_logout_pdus;	/* #of logout pdus */
	__u32 iscsi_ssn_snack_pdus;	/* #of SNACK pdus */

	__u32 iscsi_ssn_noprx_pdus;	/* #of nop PDUs rxd */
	__u32 iscsi_ssn_rsp_pdus;	/* #of rsp PDUs transferred */
	__u32 iscsi_ssn_tskmgmtrsp_pdus;	/* #of task mgmt func resp */
	__u32 iscsi_ssn_textrsp_pdus;	/* #of text resp */
	__u32 iscsi_ssn_datain_pdus;	/* #of data in -READ PDUs */
	__u32 iscsi_ssn_logoutrsp_pdus;	/* #of logout pdus */
	__u32 iscsi_ssn_r2t_pdus;	/* #of r2t rxd */
	__u32 iscsi_ssn_async_pdus;	/* #of async pdu rxd */
	__u32 iscsi_ssn_rjt_pdus;	/* #of reject pdus rxd */

	__u32 iscsi_ssn_digest_err;	/* PDUs with data/hdr digest error */
	__u32 iscsi_ssn_timeout_err;	/* connections terminated due to timeouts */

	__u64 iscsi_ssn_txdata_octets;	/* Data octets transmitted */
	__u64 iscsi_ssn_rxdata_octets;	/* Data octets received */
};

#define UPDATE_TX_SESS_STATS(opcode, command, sess_stats, connection)	\
  do {\
    __u64 txlength;\
    switch(opcode) {\
	case ISCSI_INIT_NOP_OUT:\
		sess_stats->iscsi_ssn_noptx_pdus++;\
		break;\
	case ISCSI_INIT_SCSI_CMND:\
		sess_stats->iscsi_ssn_scsicmd_pdus++;\
	 	txlength = command->tx_data_length;\
		sess_stats->iscsi_ssn_txdata_octets += txlength;\
		connection->iscsi_conn_txdata_octets += txlength;\
		break;\
	case ISCSI_INIT_TASK_MGMT_CMND:\
		sess_stats->iscsi_ssn_tskmgmtcmd_pdus++;\
		break;\
	case ISCSI_INIT_TEXT_CMND:\
		sess_stats->iscsi_ssn_text_pdus++;\
		break;\
	case ISCSI_INIT_SCSI_DATA_OUT:\
		sess_stats->iscsi_ssn_dataout_pdus++;\
	 	txlength = command->tx_data_length;\
		sess_stats->iscsi_ssn_txdata_octets += txlength;\
		connection->iscsi_conn_txdata_octets += txlength;\
		break;\
	case ISCSI_INIT_LOGOUT_CMND:\
		sess_stats->iscsi_ssn_logout_pdus++;\
		break;\
	case ISCSI_INIT_SNACK:\
		sess_stats->iscsi_ssn_snack_pdus++;\
		break;\
	default:\
		break;\
    }\
 } while(0)


#define UPDATE_RX_SESS_STATS(opcode, size, sess_stats, connection)	\
    switch(opcode) {\
	case ISCSI_TARG_NOP_IN:\
		sess_stats->iscsi_ssn_noprx_pdus++;\
		break;\
	case ISCSI_TARG_SCSI_RSP:\
		sess_stats->iscsi_ssn_rsp_pdus++;\
		break;\
	case ISCSI_TARG_TASK_MGMT_RSP:\
		sess_stats->iscsi_ssn_tskmgmtrsp_pdus++;\
		break;\
	case ISCSI_TARG_TEXT_RSP:\
		sess_stats->iscsi_ssn_textrsp_pdus++;\
		break;\
	case ISCSI_TARG_SCSI_DATA_IN:\
		sess_stats->iscsi_ssn_datain_pdus++;\
		sess_stats->iscsi_ssn_rxdata_octets += size;\
		connection->iscsi_conn_rxdata_octets += size;\
		break;\
	case ISCSI_TARG_R2T:\
		sess_stats->iscsi_ssn_r2t_pdus++;\
		break;\
	case ISCSI_TARG_ASYNC_MSG:\
		sess_stats->iscsi_ssn_async_pdus++;\
		break;\
	case ISCSI_TARG_RJT:\
		sess_stats->iscsi_ssn_rjt_pdus++;\
		break;\
	default:\
		break;\
    }


/*
 * These are what the SNMP Stats care about
 */

enum login_failure_type {
	UNKNOWN_FAILURE = 0,
	ACCEPT_RESPONSE,
	REDIRECT_RESPONSE,
	AUTH_FAIL_RESPONSE,
	AUTHENTICATE_FAIL,
	NEGOTIATE_FAIL,
	OTHER_FAIL
};

enum session_failure_type {
	UNKNOWN_ERROR = 0,
	DIGEST_ERROR,
	ISCSI_TIMEOUT_ERROR,
	FORMAT_ERROR
};

/*
 * These are some of the Instance wide stats
 */

struct iscsi_instance_stats {
	__u32 iscsi_intr_ssn_failure;	/* # time the sessions failed */
	enum session_failure_type iscsi_last_ssn_failure_type;	/* Last session failure type */
	__u32 iscsi_ssn_digest_error;	/* Sess which were failed due to receipt of a
					   PDU containing header/data digest error */
	__u32 iscsi_ssn_csn_timeout_error;	/* Sess which were failed due to sequence
						   exceeding a time limit */
	__u32 iscsi_ssn_format_error;	/* Sess which were failed due to receipt
					   of aPDU containing format error */

	__u32 iscsi_intr_login_failure;	/* # login attempt from this initiator failed */

	enum login_failure_type iscsi_intr_last_failure_type;	/* Last failure type */
	__u32 iscsi_intr_login_accept_rsp;	/* Login rsp pdus with status 0x000 */
	__u32 iscsi_intr_login_redirect_rsp;	/* Login rsp pdus with status class 0x01 */
	__u32 iscsi_intr_login_auth_fail_rsp;	/* Login rsp pdus with status class 0x201 */
	__u32 iscsi_intr_login_authenticate_fails;	/* Intr aborted because tgt couldn't be authenticated */
	__u32 iscsi_intr_login_negotiate_fails;	/* parameter neg. with tgt failed */
	__u32 iscsi_intr_login_other_fail_rsp;	/* login rsp pdus with status not defined below */

	__u32 iscsi_intr_logout_normals;	/* logout cmd pdus  with reason code 0x0 */
	__u32 iscsi_intr_logout_others;	/* logout cmd pdus with reason code other than 0 */

	struct timeval iscsi_intr_last_failure_time;	/* timestamp of most recent login failure */
	char *iscsi_intr_last_tgt_failure_name;	/* Target name */
	struct sockaddr *iscsi_intr_last_tgt_failure_address;	/* Target address */
};

#endif


/*	one of these per connection */
struct connection {
	unsigned long rx_lock_flags;	/* irqsave when rx_thread holds lock */
	unsigned long tx_lock_flags;	/* irqsave when tx_thread holds lock */
	__u32 connection_id;	/* CID within session */
	__u32 max_send_length;	/* MaxRecvDataSegmentLength from targ */
	__u32 max_recv_length;	/* MaxRecvDataSegmentLength of init */
	__u32 nop_counter;	/* counter used in NopOut ping data */
	struct session *my_session;	/* ptr to sess this conn belongs to */
	__u32 connection_flags;	/* bits for hdr/data digests, activity */
	__u32 connection_state;	/* CONNECTION_NOT_PRESENT, ... */
	struct semaphore rx_done_sem;	/* wait here for rx_thread start/stop */
	struct semaphore tx_sem;	/* tx_thread waits here for work */
	struct semaphore tx_done_sem;	/* wait here for tx_thread start/stop */

	__u32 nop_remaining;	/* no. of secs until next NopOut sent */
	atomic_t tx_timer_went_off;	/* no. of pending timer expirations */
	struct timer_list tx_timer;	/* timer for sending nops */

	/* the following field is manipulated by add_(remove_)session only */
	struct connection *next;	/* list of connections in this session */

	struct socket *sock;	/* pointer to open socket to target */

	/* the following 2 fields are only set up after login succeeds */
	struct task_struct *rx_thread;	/* pointer to TCB of rx_thread */
	struct task_struct *tx_thread;	/* pointer to TCB of tx_thread */

	int rec_tests;		/* Indicator to initiate conn recovery SAI */

	/* the following 5 fields are used by the rx_thread during operation */

	struct list_head pending_commands;	/* SCSI commands active on this conn */
	struct msghdr rx_msg;	/* header for sock_recvmsg on this conn */
	struct iovec rx_iov[MAX_IOV_SLOTS];	/* recvmsg iovec built here */
	struct iovec rx_iov_copy[MAX_IOV_SLOTS];	/* copy of rx_iov if digest */
	struct list_head r2t_list;	/* threaded order to send r2ts */

#if defined(CONFIG_ISCSI_DEBUG)
	struct iovec *iov_copy;	/* "local" to recv_iovector() */
#endif

	/* following 2 fields are used by tx_thread during operation */
	struct command *short_command;	/* Non-null if partially sent command */
	__u32 process_r2t_order;	/*next r2t to process in order on conn */

	struct sockaddr *ip_address;	/* holds target's ipv4/ipv6 address */
	int ip_length;		/* used no. of bytes of ip_address */
	struct sockaddr *local_ip_address;	/* holds init's ipv4/ipv6 address */

	/* the following field is only used for CONN_SCHED_LUN scheduling */
	__u32 assigned_lun_count;	/* no. of luns assigned to this conn */

	__u32 basic_hdr_len;	/* includes CRC_LEN as necessary */

	/* fields used to update ExpStatSN correctly */
	__u32 last_rcvd_stat_sn;	/* last StatSN received from target */
	__u32 exp_stat_sn;	/* next ExpStatSN to send to target */
	__u32 outstanding_requests;	/* number of response pdus expected */
	int last_accepted;	/* array index to last accepted resp */
	int max_accepted;	/* number of bytes in following arrays */
	char *isvalid;		/* array of flags: 1 if valid else 0 */
	__u32 *accepted_stat_sn;	/* array of accepted resp stat_sn */

#ifdef ISCSI_STATS
	struct proc_dir_entry *proc_dir;	/* Proc dir entry */
	struct proc_dir_entry *proc_stat_file;	/* Proc stat file */
#ifdef USE_CONTROL_FILE
	struct proc_dir_entry *proc_ctrl_file;	/* Proc ctrl file */
#endif
	__u64 iscsi_conn_txdata_octets;	/* Data octets transmitted */
	__u64 iscsi_conn_rxdata_octets;	/* Data octets received */
#endif
	__u8 *rx_buf;

#ifdef RDMA_MODE
	/* received PDU queue */
	struct queue rx_pdu_q;

	struct iser_recv_pdu_t *rx_pdu_front;

	/* Added for iSER - Minti */
	/*int iser_conn_id; Integer that uniquely identifies this connection to 
	   (across sessions) to iSER */
	struct semaphore iser_notify_sem;	/* wait for recv PDU's and conn terminations */
	struct semaphore iser_conn_sem;	/* wait for connection establishments */
	struct semaphore send_done_sem;
#endif
};

/*	one of these structures for each initiator session */
struct session {
	struct list_head link;	/* for threaded-list of sessions */
	UNH_LOCK_TYPE sess_lock;	/* spin lock for each session */

	/* the following fields are used to store free command structures */
	struct list_head free_commands;
	__u16 n_commands_alloced;	/* no. of commands alloced to session */
	__u16 n_luns_in_session;	/* no. of active luns in session */

	/* the following field is used to store free r2t cookies */
	struct list_head free_r2t_cookies;

	__u32 scsi_target_id;	/* target in [host,bus,target,lun] */
	__u32 lun_bits;		/* bits for lun in [host,bus,target,lun] */
	__u32 session_state;	/* SESSION_NOT_PRESENT, ... */
	__u32 cur_task_tag;	/* generator for unique ITTs */
	__u32 cur_cmd_sn;	/* generator for sequential CmdSNs */
	__u32 exp_cmd_sn;	/* last ExpCmdSN from target */
	__u32 max_cmd_sn;	/* last MaxCmdSN from target */

	struct timer_list connrec_timer;	/* Time2wait timer - SAI */

	/* the following fields are initialized from configured hostdata */
	__u32 session_flags;	/* flags for session-wide control (none yet) */
	__u32 init_conn_flags;	/* initial connection-specific flags */
	__u32 nop_period;	/* no. of secs between sending NopOuts */
	__u32 init_snack_flag;	/* Store SNACK Related flags here - SAI */
	__u32 retran_period;	/* retransmission threshold in sec - SAI */
	__u32 sched_scheme;	/* connection scheduling scheme to use - SAI */

	struct parameter_type (*session_params)[MAX_CONFIG_PARAMS];

#ifdef ISCSI_STATS
	struct proc_dir_entry *proc_dir;	/* Proc dir entry */
	struct proc_dir_entry *proc_stat_file;	/* Proc stat file */
#ifdef USE_CONTROL_FILE
	struct proc_dir_entry *proc_ctrl_file;	/* Proc ctrl file */
#endif
	struct iscsi_session_stats *sess_stats;	/* per session Stats */
#endif

	/* the following 2 fields are manipulated by add_(remove_)session only */
	__u32 nconnections;	/* no. of connections in next list */
	struct connection *connection_head;	/* list of connections in this sess */

	struct connection *rr_conn_head;	/* round robin connection head SAI */

	/* the following field is filled in after login finishes successfully */
	struct session_operational_parameters oper_param;

	/* the following field holds the configurable SCSI Command ATTR byte */
	__u8 dummy1;		/* to preserve alignment */
	__u8 dummy2;		/* to preserve alignment */

	/* the following field is returned by target if first login succeeds */
	__u16 tsih;		/* target's handle for this session */

	int n_rec_tasks;	/* stores no. of tasks that were transfered from
				 * a failing connection - SAI
				 */
	/* the following table is used only for CONN_SCHED_LUN scheduling */
	/* index by LUN to find assigned connection */
	struct connection *lun_assignments[MAX_LUNS];

	/* the following 2 arrays are used to keep track of inquiry data */
	/* index by LUN to get latest inquiry info */
	__u8 *inquiry_buf[MAX_LUNS];	/* buffer to hold inquiry data */
	int inquiry_size[MAX_LUNS];	/* no of bytes alloced to inquiry_buf */

	/* the following 2 arrays are used to keep track of read capacity data */
	/* index by LUN to get latest read capacity info */
	__u32 capacity_lba[MAX_LUNS];	/* returned logical block address */
	__u32 capacity_len[MAX_LUNS];	/* block length in bytes */

	/* the following 2 arrays are used to keep track of block limits data */
	/* index by LUN to get latest block limits info */
	__u32 limit_max[MAX_LUNS];	/* max block length limit */
	__u32 limit_min[MAX_LUNS];	/* min block length limit */

	/* the following array holds the ATTR value for each lun */
	__u8 lun_attr[MAX_LUNS];	/* ATTR set from INQUIRY data */
};

/*	Target specific configuration data */
struct iscsi_targetdata {
	__u32 nop_period;	/* timer value to force NopOuts */
	__u32 init_sess_flags;	/* initial session-wide flags */
	__u32 init_conn_flags;	/* initial connection-specific flags */
	__u8 init_snack_flag;	/* holds SNACK related flags - SAI */

	__u8 attr;		/* ATTR field in SCSI Command, range is 0 - 7 */
	__u8 dummy1;		/* to preserve alignment */
	__u8 dummy2;		/* to preserve alignment */

	__u32 isid_type;	/* holds isid type, max of 0x03 */
	__u32 isid_number;	/* holds isid number, max of 0xffffff */
	__u32 isid_qualifier;	/* holds isid qualifier, max of 0xffffff */

	__u32 retran_period;	/* holds rexmit threshold in sec SAI */
	__u32 sched_scheme;	/* holds scheduling algorithm to use
				   with multiple connections - SAI */
	/* chap support -CHONG */
	struct auth_parameter_type auth_parameter;

	struct parameter_type param_tbl[MAX_CONFIG_PARAMS];
};


#ifndef __KERNEL__
/* skeleton from /usr/src/linux/include/scsi/scsi_device.h */
struct scsi_device {
	struct Scsi_Host *host;
	unsigned int id, lun;
};
#endif


/*	One of these structures forms the private data part for each
 *	iscsi adaptor.  Since we have only 1 such adaptor, we will
 *	instantiate this structure only once, when the iscsi_initiator
 *	module is first loaded.  At that time the variable global_hostdata
 *	will be set to point to the instantiation, and the fields will be
 *	set up to the defaults (all 0 except for param_tbl, which gets a copy
 *	of config_params[]).  After that, whenever the user changes parameter
 *	values using iscsi_manage, those changes are made to the values in this
 *	structure.
 */
struct iscsi_hostdata {
	__u32 nsessions;	/* no. of targets/sessions so far */
	struct list_head session_list;	/* list of threaded struct sessions */

	/* recovery session list - SAI */
	struct sess_rec_group *sess_rec_head;

	int init_sessrec_flg;	/* 1 if sess recovery in progress SAI */
	/* puts scsi to sleep in sess recovery - SAI */
	struct semaphore init_sessrec_sem;
	struct iscsi_targetdata target_data[MAX_TARGET_IDS];
#ifdef K26			/* for 2.6.0 hot-plug HBA model */
	struct list_head host_list;
	struct device dev;
	struct Scsi_Host *shost;
#endif
#ifdef ISCSI_STATS
	struct iscsi_instance_stats instance_stats;	/* Login statistics */
#endif
#ifdef RDMA_MODE
	/* Added for iSER - Minti */
	iser_api_handle_t iser_handle;
	struct iser_api_t iser_api;
	struct iser_ext_api_t iser_ext_api;
	int iser_conn_ctr;	/* Integer that increments for every iSER connection 
				   across sessions */
#endif
};

/* Add the Error Recovery Structure - SAI */

struct init_error_rec {
	struct connection *curr_conn;
	struct command *related_cmd;
	__u8 err_type;
};

/*******************************************************************/
/*	the following functions are defined in initiator_proc_ioface.c */
/*******************************************************************/

/* Ming Zhang, mingz@ele.uri.edu */
#ifdef K26
int iscsi_initiator_proc_info(struct Scsi_Host *, char *, char **, off_t,
			      int, int);
#else
int iscsi_initiator_proc_info(char *, char **, off_t, int, int, int);
#endif

/**********************************************************/
/*	the following functions are defined in initiator_rx.c */
/**********************************************************/

#ifndef __KERNEL__
/* cleanup handler called if rx_thread is cancelled while sitting on readv() */
void rx_readv_cleaner(void *ptr);
#endif

void iscsi_initiator_rx_thread(void *arg);

/**********************************************************/
/*	the following functions are defined in initiator_tx.c */
/**********************************************************/

/*	Called to send a NopOut to target as:
 *		a new nop (nopin_cmd == NULL, size == 0 )
 *	or
 *		a new ping (nopin_cmd == NULL, size != 0 )
 *	or
 *		a response to a NopIn (nopin_cmd != NULL).
 *	Buffer points to data to send, size is number of bytes of data to send.
 *	If size > 0 then better have buffer != NULL.
 *	Returns 1 if all ok, else 0.
 */
int tx_nopout(struct connection *current_connection,
	      __u32 i_bit,
	      struct iscsi_targ_nopin *nopin_cmd, char **buffer, int size);

/*	drives sending of a nop out to the target */
int drive_nopout(struct connection *conn,
		 struct session *current_session, __u32 i_bit,
		 __u32 want_data);

/* Values for nego_type parameter to drive_text_negotiation */
enum text_nego_type { TARGET_TEXT_NEGO, DISCO_TEXT_NEGO, INI_TEXT_NEGO };

/*	drives the discovery session from within the tx_thread
 *	returns 0 on success, 1 on failure
 */
int drive_text_negotiate(struct connection *conn,
			 struct session *current_session,
			 __u32 i_bit, enum text_nego_type nego_type);

/*	drives the logout of a session from within the tx_thread
 *	returns 1 on success, 0 on failure
 *	Modified to include connection logout - SAI
 */
int drive_logout(struct session *current_session, struct connection *conn,
		 int reason);

void iscsi_initiator_tx_thread(void *);

/*************************************************************
 *  the following functions are defined in iscsi_initiator.c *
 *************************************************************/

/* initialize all fields in struct iscsi_targetdata for one target */
void init_targetdata(struct iscsi_targetdata *targ_data);

/*	find a lun that is already in use in this session
 *	returns -1 on error, else lun number in range [0..31]
 */
int find_used_lun(struct session *sess);

/* uninitialize fields in struct iscsi_targetdata that need freeing */
void uninit_targetdata(struct iscsi_targetdata *targ_data);

/* Create a new recovery connection to keep the iSCSI session active.
 * Remember, iscsi requires at least one connection in a session to be active
 * for the session to be alive - SAI
 * Returns 1 on success, 0 on failure.
 */
int create_rec_conn(struct connection *conn);

void __attribute__ ((no_instrument_function))
    clear_free_conn(struct session *sess);

/* Declaration needed as this function used by Error Recovery - SAI */
struct connection * __attribute__ ((no_instrument_function))
    get_connection(struct scsi_cmnd *Cmnd, struct session *current_session);

int create_session(__u32 target, struct sockaddr *ip_address,
		   int ip_length, __u32 lun, __u32 cid,
		   struct Scsi_Host *host);

int remove_session_from_hostdata(__u32 target, __u32 lun, __u32 cid,
				 struct iscsi_hostdata *hostdata);

int find_connection(__u32 target,
		    __u32 cid,
		    struct iscsi_hostdata *hostdata,
		    struct connection **old_connection,
		    struct session **old_session);

/*	Mike Christie, mikenc@us.ibm.com
 *
 * remove all valid luns in session (last lun will take out session)
 * returns smallest lun number removed, -1 if none removed.
 */
int
remove_luns(struct session *sess, struct Scsi_Host *host,
	    struct iscsi_hostdata *hostdata);

/* SAI */
int scsi_to_iscsi(struct scsi_cmnd *Cmnd, struct session *current_session);

#ifdef CONFIG_ISCSI_DEBUG
int __get_cpu_number(void);
#endif

int max_outstanding_commands(void);


/* From iscsi_device.c */
extern int initiator_register_device(void);
extern void initiator_unregister_device(void);

#ifdef K26
static inline int __attribute__ ((no_instrument_function))
    use_sendpage(struct connection *conn, struct command *cmnd)
{
/* TODO - move to use of sendpage() */
#if 0
	if (!(conn->connection_flags & USE_DATADIGEST)
	    && cmnd->SCpnt != NULL && cmnd->SCpnt->use_sg > 0)
		return 1;
	else
#endif
		return 0;
}
#endif

#if !defined(list_for_each_entry)

/* include/linux/list.h is out of date */

/**
 * list_for_each_entry	-	iterate over list of given type
 * @pos:	the type * to use as a loop counter.
 * @head:	the head for your list.
 * @member:	the name of the list_struct within the struct.
 */
#define list_for_each_entry(pos, head, member)				\
	for (pos = list_entry((head)->next, typeof(*pos), member),	\
		     prefetch(pos->member.next);			\
	     &pos->member != (head); 					\
	     pos = list_entry(pos->member.next, typeof(*pos), member),	\
		     prefetch(pos->member.next))

#endif

/*
 * when called, host_data_lock MUST be held by calling thread/process.
 * searches the session_list pointed to by hostdata to try to find
 * a session with id that matches target_id
 * Returns pointer to that session if found, NULL if not found
 */
static inline struct session * __attribute__ ((no_instrument_function))
    find_session_by_id(__u32 target_id, struct iscsi_hostdata *hostdata)
{
	struct session *sess;

	list_for_each_entry(sess, &hostdata->session_list, link) {
		if (sess->scsi_target_id == target_id) {
			return sess;
		}
	}

	/* if loop finishes, no session with this id was found */
	return NULL;
}

static inline struct connection * __attribute__ ((no_instrument_function))
    find_conn_by_iser_conn_handle(void *iser_conn_handle)
{
	struct connection *conn, *ret_conn = NULL;
	struct session *sess;

	list_for_each_entry(sess, &global_hostdata->session_list, link) {
		for (conn = sess->connection_head; conn != NULL;
		     conn = conn->next) {
			TRACE(TRACE_ISER,
			      "find_conn_by_iser_conn_handle: sess%p conn%p\n",
			      sess, conn);
			if (conn == iser_conn_handle) {
				ret_conn = conn;
				break;
			}
		}
	}
	TRACE(TRACE_ISER, "Leave find_conn_by_iser_conn_handle: conn%p\n",
	      ret_conn);
	return ret_conn;
}

/* called only during login phase
 * to deal with ExpCmdSN and MaxCmdSN in Login Response PDU
 */
int __attribute__ ((no_instrument_function))
    lock_and_check_and_update_cmd_sns(struct iscsi_targ_login_rsp *pdu,
				  struct session *session);

#endif				/* _ISCSI_INITIATOR_H */
