/*	target/iscsi_target_module.c
 *
 *   	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 * 	This file must be included by every front-end target driver that is
 * 	trying to register with the mid-level. This is the common portion
 * 	that gets included with all the low-level drivers.
 *
 * 	To use this the low-level driver must define the variable my_template
 * 	of the type Scsi_Target_Template with all the needed functions and
 * 	values defined in scsi_target.h
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 *	written permission.
*/

#include "scsi_target.h"
#include "../common/debug.h"

/* From target/iscsi_target.c */
extern Scsi_Target_Template my_template;

int iscsi_target_module_init(void)
{
#if defined(RDMA_MODE)
	extern int iser_init_module(void);

	if (iser_init_module() < 0)
		TRACE_ERROR("iser_init_module failed");
#endif				/* RDMA_MODE */

	debugInit("iscsi_target_module_init: Initializing: %s\n",
		  my_template.name);
	if (register_target_template(&my_template) < 0)
		return (-1);

	return (0);
}

void iscsi_target_module_cleanup(void)
{
#if defined(RDMA_MODE)
	extern void iser_cleanup_module(void);
#endif /* RDMA_MODE */

	debugInit("iscsi_target_module_cleanup enter\n");

	if (deregister_target_template(&my_template) < 0)
		printk("Error in the deregistration process\n");

#if defined(RDMA_MODE)
	iser_cleanup_module();
#endif				/* RDMA_MODE */

	debugInit("iscsi_target_module_cleanup exit\n");
}

module_init(iscsi_target_module_init);
module_exit(iscsi_target_module_cleanup);
