/*	target/scsi_request.h
 *
 *	vi: set autoindent tabstop=8 shiftwidth=8 :
 *
 *	Copyright (C) 2001-2007 InterOperability Lab (IOL)
 *	University of New Hampshire (UNH)
 *	Durham, NH 03824
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *	USA.
 *
 *	The name of IOL and/or UNH may not be used to endorse or promote
 *	products derived from this software without specific prior
 *	written permission.
*/


#ifndef __SCSI_REQUEST_H__
#define __SCSI_REQUEST_H__

#ifdef __KERNEL__

#include <scsi/scsi_cmnd.h>	/* MAX_COMMAND_SIZE - eaburns */

#else

/* from /usr/src/linux/include/scsi/scsi.h */
/*
 * Host byte codes
 */

#define	DID_OK		0x00	/* NO error     */


/* from /usr/src/linux/include/scsi/scsi_cmnd.h */
#define	MAX_COMMAND_SIZE	16
#define SCSI_SENSE_BUFFERSIZE	96

/* from /usr/src/linux/include/linux/dma-mapping.h */
/*
enum dma_data_direction {
	DMA_BIDIRECTIONAL = 0,
	DMA_TO_DEVICE = 1,
	DMA_FROM_DEVICE = 2,
	DMA_NONE = 3,
};
*/

#ifndef SCSI_DATA_UNKNOWN
#define SCSI_DATA_UNKNOWN	(DMA_BIDIRECTIONAL)
#endif
#ifndef SCSI_DATA_WRITE
#define SCSI_DATA_WRITE		(DMA_TO_DEVICE)
#endif
#ifndef SCSI_DATA_READ
#define SCSI_DATA_READ		(DMA_FROM_DEVICE)
#endif
#ifndef SCSI_DATA_NONE
#define SCSI_DATA_NONE		(DMA_NONE)
#endif

#endif

/*
 * >=2.6.18 does not have Scsi_Request structure.  We define our own here to
 * allow for MEMORYIO and FILEIO to simulate the scsi midlevel the same way as
 * it did pre-2.6.18
 *
 * This structure is also used in DISKIO mode so that we can use a
 * re-implementation of scsi_do_req() that takes the same (simular) parameters
 * as pre-2.6.18 versions did.
 *
 * - eaburns
 */

/* skeleton from /usr/src/linux/include/scsi/scsi_cmnd.h */
#ifndef __KERNEL__
struct scsi_cmnd {
	int resid;		/* Number of bytes requested to be
				   transferred less actual number
				   transferred (0 if not supported) */
};
#endif

/*
 * A stripped down struct scsi_request that contains only the fields that are
 * used to emulate the scsi midlevel.
 *
 * (field comments copied from scsi/scsi_request.h in linux-2.6.17.10)
 */
struct scsi_request {
	int sr_result;		/* Status code from lower level driver */

	/* Obtained by REQUEST SENSE when CHECK CONDITION is received on
	 * original command (auto-sense) */
	unsigned char sr_sense_buffer[SCSI_SENSE_BUFFERSIZE];

	/*
	 * In DISKIO mode, this field needs to be filled in so that
	 * we can use it in our re-implementation of scsi_do_req().
	 * - eaburns
	 */
	struct scsi_device *sr_device;

	struct scsi_cmnd *sr_command;
	unsigned sr_bufflen;	/* Size of data buffer */
	void *sr_buffer;	/* Data buffer */
	int sr_allowed;
	enum dma_data_direction sr_data_direction;
	unsigned char sr_cmnd[MAX_COMMAND_SIZE];
	unsigned short sr_use_sg;	/* Number of pieces of scatter-gather */
	unsigned short sr_sglist_len;	/* size of malloc'd scatter-gather list */
};
typedef struct scsi_request Scsi_Request;

/*
 * declarations for functions that were re-implemented after the removal of
 * the scsi_request structure.
 */
void scsi_do_req(struct scsi_request *sreq, const unsigned char *cmnd,
		 void *buffer, unsigned bufflen,
		 void (*done) (void *, char *, int, int),
		 int timeout, int retries);
struct scsi_request *scsi_allocate_request(struct scsi_device *sdev,
					   gfp_t gfp_mask);
void scsi_release_request(struct scsi_request *sreq);

#endif
